package org.ziggurat.fenix.common.database.hibernate;

import java.io.Serializable;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;

public class HibChequesRepository extends GenericHibernateRepository<Cheque, Long> implements Serializable  {

	@Override
	public List<Cheque> findAll() {
		List<Cheque> lista;
		Session session = HibernateUtil.currentSession();
		Criteria criCheques = session.createCriteria(Cheque.class)
			.add(Restrictions.eq("eliminado", false))
			.addOrder(Order.asc("fecha"));
		return criCheques.list();
	}

	public List<Cheque> getChequesPorEstado(EstadoDeCheque estado) {
		Session session = HibernateUtil.currentSession();
		Criteria criCheques = session.createCriteria(Cheque.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.eq("estado", estado))
			.addOrder(Order.asc("fecha"));
		return criCheques.list();
	}
	
	public List<Cheque> getPaginaDeChequesPorEstado(EstadoDeCheque estado, int start, int count) {
		Session session = HibernateUtil.currentSession();
		Criteria criCheques = session.createCriteria(Cheque.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.eq("estado", estado))
			.addOrder(Order.asc("fecha"))
			.setFirstResult(start)
			.setMaxResults(count);
		return criCheques.list();
	}
	
	public int countCheques(EstadoDeCheque estado) {
		Session session = HibernateUtil.currentSession();
		return (Integer)session.createCriteria(Cheque.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.eq("estado", estado))
			.setProjection(Projections.rowCount())
			.uniqueResult();
	}

	public List<Cheque> findByExample(Cheque exampleInstance, String... excludeProperty) {
		throw new IllegalStateException("Metodo no implementado");
	}
	
	public void eliminarCheque(Cheque cheque) {
		cheque.setEliminado(true);
		makePersistent(cheque);
	}
	
}
