package org.ziggurat.fenix.common.database.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;

public class HibClienteRepository extends GenericHibernateRepository<Cliente, Long> {

	@Override
	public List findAll() {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Cliente.class)
			.addOrder(Order.asc("nombre"))
			.list();
	}
	
	public List<Cliente> findListaDeClientesPage(int start, int count) {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Cliente.class)
			.addOrder(Order.asc("nombre"))
			.setFirstResult(start)
			.setMaxResults(count)
			.list();
	}
	
	public List<Cliente> findListaDeClientesPage(int start, int count, String busqueda) {
		String nombreLike = "%" + busqueda + "%";
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Cliente.class)
			.add(Restrictions.ilike("nombre", nombreLike))
			.addOrder(Order.asc("nombre"))
			.setFirstResult(start)
			.setMaxResults(count)
			.list();
	}
	
	public int countClientesFiltrados(String busqueda) {
		String nombreLike = "%" + busqueda + "%";
		Session session = HibernateUtil.currentSession();
		return (Integer)session.createCriteria(Cliente.class)
			.add(Restrictions.ilike("nombre", nombreLike))
			.setProjection(Projections.rowCount())
			.uniqueResult();
	}
	
	
	public int countClientes() {
		Session session = HibernateUtil.currentSession();
		return (Integer) session.createCriteria(Cliente.class)
			.setProjection(Projections.rowCount())
			.uniqueResult();
	}
	
	public List<Cliente> getClientesPorNombre(String nombre) {
		Session session = HibernateUtil.currentSession();
		String nombreLike = "%" + nombre + "%";
		Criteria cri = session.createCriteria(Cliente.class)
			.add(Restrictions.ilike("nombre", nombreLike));
		return cri.list();
	}

}
