package org.ziggurat.fenix.common.database.hibernate;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.time.DateUtils;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.ziggurat.fenix.common.modelo.proveedores.CategoriaDeProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.CompraAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.LibroDeIvaCompras;
import org.ziggurat.fenix.common.modelo.proveedores.PagoAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.ResumenMensual;
import org.ziggurat.fenix.common.modelo.proveedores.ResumenPorTipo;
import org.ziggurat.fenix.common.modelo.proveedores.TransaccionConProveedor;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;
import org.ziggurat.fenix.common.modelo.transacciones.TipoDeTransaccion;

public class HibTransaccionConProveedorRepository extends GenericHibernateRepository<TransaccionConProveedor, Long> {

	public TransaccionConProveedor makePersistentAndUpdateCheque(PagoAProveedorInmutable entity) {
		PagoAProveedorInmutable pago = (PagoAProveedorInmutable) entity;
		pago.getCheque().setEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR);
		HibChequesRepository repoCheque = new HibChequesRepository();
		repoCheque.makePersistent(pago.getCheque());
		return super.makePersistent(entity);
	}
	
	public List<TransaccionConProveedor> findAll() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<TransaccionConProveedor> findByExample(TransaccionConProveedor exampleInstance, String... excludeProperty) {
		// TODO Auto-generated method stub
		return null;
	}
	
	public void eliminarTransaccion(TransaccionConProveedor tr) {
		Session session = HibernateUtil.currentSession();
		if (tr.getTipoDeTransaccion().equals(TipoDeTransaccion.PAGO_A_PROVEEDOR_CHEQUE)) {
			PagoAProveedorInmutable pago = (PagoAProveedorInmutable) tr;
			pago.getCheque().setEstado(EstadoDeCheque.EN_POSESION_DE_FENIX);
			HibChequesRepository repoCheque = new HibChequesRepository();
			repoCheque.makePersistent(pago.getCheque());
		}
		tr.setEliminado(true);
		makePersistent(tr);
	}
	
	public BigDecimal getGastoDelMes(CategoriaDeProveedor tipo, int mes, int ano) throws ParseException {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		
		Session session = HibernateUtil.currentSession();
		Criteria cri = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.createAlias("proveedor", "prov")
			.add(Restrictions.eq("prov.categoria", tipo))
			.add(Restrictions.between("fecha", fechaInicio, fechaFin));
		cri.setProjection(Projections.sum("total"));
		BigDecimal gasto = (BigDecimal) cri.uniqueResult();
		if (gasto == null) 
			return BigDecimal.ZERO;
		else
			return gasto;
	}
	
	public BigDecimal getGastoDelMes(int mes, int ano) throws ParseException {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		
		Session session = HibernateUtil.currentSession();
		Criteria cri = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", fechaInicio, fechaFin));			
		cri.setProjection(Projections.sum("total"));
		BigDecimal gasto = (BigDecimal) cri.uniqueResult();
		if (gasto == null) 
			return BigDecimal.ZERO;
		else
			return gasto;
	}
	
	public List<PagoAProveedorInmutable> getPagosDelMes(int mes, int ano) {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = null;
		try {
			fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		
		Session session = HibernateUtil.currentSession();
		Criteria cri = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", fechaInicio, fechaFin))
			.addOrder(Order.desc("fecha"));
		List<PagoAProveedorInmutable> pagos = cri.list();		
		return pagos;
	}
	
	private ResumenPorTipo getResumenPorTipoSinCerrarTransaccion(int mes, int ano, CategoriaDeProveedor tipo) throws ParseException {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		
		Session session = HibernateUtil.currentSession();
		Criteria critCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.createAlias("proveedor", "prov")
			.add(Restrictions.eq("prov.categoria", tipo))
			.add(Restrictions.between("fecha", fechaInicio, fechaFin))
			.addOrder(Order.desc("fecha"));
		List<CompraAProveedorInmutable> compras = critCompras.list();
		Criteria critPagos = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.createAlias("proveedor", "prov")
			.add(Restrictions.eq("prov.categoria", tipo))	
			.add(Restrictions.between("fecha", fechaInicio, fechaFin))
			.addOrder(Order.desc("fecha"));
		List<PagoAProveedorInmutable> pagos = critPagos.list();
		
		ResumenPorTipo resumen = new ResumenPorTipo(tipo, ano, mes, compras, pagos);
		return resumen;
	}

	public ResumenMensual getResumenProveedores(int mes, int ano) {
		HibProveedorRepository repoProv = new HibProveedorRepository();
		HibCategoriasDeProveedoresRepository repoCat = new HibCategoriasDeProveedoresRepository();
		List<ResumenPorTipo> listaResumenes = new ArrayList<ResumenPorTipo>();
		Session session = HibernateUtil.currentSession();
		//session.beginTransaction();
		for (CategoriaDeProveedor tipo : repoCat.findAll()) {
			try {
				ResumenPorTipo resumen = getResumenPorTipoSinCerrarTransaccion(mes, ano, tipo);
				listaResumenes.add(resumen);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		ResumenMensual resumen = new ResumenMensual(listaResumenes);
		//session.getTransaction().commit();
		return resumen;
	}
	
	public ResumenPorTipo getResumenPorTipo(int mes, int ano, CategoriaDeProveedor tipo) throws ParseException {
		HibernateUtil.beginTransaction();
		ResumenPorTipo resumen = getResumenPorTipoSinCerrarTransaccion(mes, ano, tipo);
		HibernateUtil.commitTransaction();		
		return resumen;
	}
	
	public List<CompraAProveedorInmutable> getComprasBetweenDates(final Date fechaInicial, final Date fechaFinal) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria cri = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", fechaInicial, fechaFinal))
			.addOrder(Order.asc("fecha"));
		return cri.list();
	}
	
	public List<CompraAProveedorInmutable> getComprasBetweenDates(final Date fechaInicial, final Date fechaFinal, CategoriaDeProveedor categoria) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria cri = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.createAlias("proveedor", "prov")
			.add(Restrictions.eq("prov.categoria", categoria))
			.add(Restrictions.between("fecha", fechaInicial, fechaFinal))
			.addOrder(Order.asc("fecha"));			
		return cri.list();
	}
	
	public List<PagoAProveedorInmutable> getPagosBetweenDates(final Date fechaInicial, final Date fechaFinal) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria cri = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", fechaInicial, fechaFinal));
		return cri.list();
	}
	
	public LibroDeIvaCompras getLibroDeIvaCompras(int mes, int ano) {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = null;
		try {
			fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		List<CompraAProveedorInmutable> compras = getComprasBetweenDates(fechaInicio, fechaFin);
		return new LibroDeIvaCompras(ano, mes, compras);
		
		/*
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = null;
		try {
			fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		
		Session session = HibernateUtil.getCurrentSession();
		session.beginTransaction();
		Query queryFacturas = session
				.createQuery("from FacturaDeProveedor as factura where factura.vigente = true and factura.fecha >= :fechaInicio and factura.fecha <= :fechaFin order by factura.fecha asc");
		queryFacturas.setDate("fechaInicio", fechaInicio);
		queryFacturas.setDate("fechaFin", fechaFin);
		List<FacturaDeProveedor> facturas = queryFacturas.list();
		LibroDeIvaCompras libro = new LibroDeIvaCompras(ano, mes, facturas);
		session.getTransaction().commit();
		return libro;
		*/
	}
	
	public LibroDeIvaCompras getLibroDeIvaCompras(int mes, int ano, CategoriaDeProveedor categoria) {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = null;
		try {
			fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		List<CompraAProveedorInmutable> compras = getComprasBetweenDates(fechaInicio, fechaFin, categoria);
		return new LibroDeIvaCompras(ano, mes, compras);
	}

}
