package org.ziggurat.fenix.common.database.hibernate;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;

public class HibernateUtil {

	private static final SessionFactory sessionFactory;

	static {
		try {
			AnnotationConfiguration annConfig= new AnnotationConfiguration();
			sessionFactory = annConfig.configure().buildSessionFactory();
		} catch (Throwable ex) {
			ex.printStackTrace();
			throw new ExceptionInInitializerError(ex);
		}
	}
	
	private static final ThreadLocal<Session> session = new ThreadLocal<Session>();
	   
    public static Session currentSession() {
      Session s = (Session) session.get();
      if (s == null) {
         SessionFactory sf = HibernateUtil.getSessionFactory();
         s = sf.openSession();
         //System.out.println("Opening session...");
         session.set(s);
      }
      return s;
    }

    public static void closeSession() throws HibernateException {
       Session s = (Session) session.get();
       session.set(null);
       if (s != null) {
    	   s.close();
    	   //System.out.println("Closing session");
       }
    }
	
	private static SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	public static long getSessionsOpened() {
		return sessionFactory.getStatistics().getSessionOpenCount();
	}
	
	public static void beginTransaction() {
		currentSession().beginTransaction();
	}
	
	public static void commitTransaction() {
		if (currentSession().getTransaction().isActive()) {
			currentSession().getTransaction().commit();
		}
	}
	
	/*
	public static void closeSession() {
		sessionFactory.getCurrentSession().getTransaction().commit();
		sessionFactory.getCurrentSession().close();
	}*/
	
	public static void deleteAllTransacciones() {
		currentSession().beginTransaction();
		String hqlDelete = "delete TransaccionConProveedor";
		currentSession().createQuery(hqlDelete).executeUpdate();
		currentSession().getTransaction().commit();
	}
	
	public static void deleteAllProveedores() {
		currentSession().beginTransaction();
		String hqlDelete = "delete Proveedor";
		currentSession().createQuery(hqlDelete).executeUpdate();
		currentSession().getTransaction().commit();
	}
	
	public static void deleteAllCheques() {
		currentSession().beginTransaction();
		String hqlDelete = "delete Cheque";
		currentSession().createQuery(hqlDelete).executeUpdate();
		currentSession().getTransaction().commit();
	}
	
	public static void deleteAllClientes() {
		currentSession().beginTransaction();
		String hqlDelete = "delete Cliente";
		currentSession().createQuery(hqlDelete).executeUpdate();
		currentSession().getTransaction().commit();
	}
	
	public static void deleteAllMatafuegos() {
		currentSession().beginTransaction();
		String hqlDelete = "delete Matafuego";
		currentSession().createQuery(hqlDelete).executeUpdate();
		currentSession().getTransaction().commit();
	}
	
}