package org.ziggurat.fenix.common.modelo.clientes;

import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

@Entity
@Inheritance(strategy=InheritanceType.JOINED)
@Table(name = "transaccion_con_cliente")
public abstract class TransaccionConCliente implements Comparable<TransaccionConCliente> {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	
	@ManyToOne( cascade = {CascadeType.PERSIST, CascadeType.MERGE} )
	private Cliente cliente;
	
	public Cliente getCliente() {
		return cliente;
	}
	
	protected void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}
	
	abstract public Date getFecha();
	abstract public BigDecimal getMonto();
	
	/**
	 * Este metodo es un metodo utilitario para ayudar a calcular
	 * saldos parciales. Se le pasa un saldo como parametro
	 * y dependiendo del tipo de movimiento, se retorna
	 * el valor del nuevo saldo con el movimiento aplicado.
	 * @param saldo
	 * @return
	 */
	abstract protected BigDecimal resolverSaldo(BigDecimal saldo);

	@Override
	public int compareTo(TransaccionConCliente o) {
		return getFecha().compareTo(o.getFecha());
	}
		
}
