package org.ziggurat.fenix.common.modelo.clientes;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

import org.ziggurat.fenix.common.modelo.comprobantes.factura.FacturaInmutable;
import org.ziggurat.fenix.common.util.MoneyHelper;

public class VentaACliente {

	private Date fecha;
	private FacturaInmutable factura;
	private Set<PagoDeClienteInmutable> pagos = new TreeSet<PagoDeClienteInmutable>();
	private BigDecimal montoEnCC;	
	private Cliente cliente;
	private int tieneChequeAsociado = 0;
	
	public VentaACliente(FacturaInmutable factura) {
		if (factura == null) throw new NullPointerException("La factura no puede ser null");
		this.factura = factura;
		montoEnCC = factura.getMonto();
	}
	
	public Date getFecha() {
		return fecha;
	}
	public void setFecha(Date fecha) {
		this.fecha = fecha;
	}
	public FacturaInmutable getFactura() {
		return factura;
	}
	public BigDecimal getMontoEnCC() {
		return montoEnCC;
	}
	public Cliente getCliente() {
		return cliente;
	}
	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}
	public Set<PagoDeClienteInmutable> getPagos() {
		return pagos;
	}
	public void addPago(PagoDeClienteInmutable pago) {		
		this.pagos.add(pago);
		montoEnCC = MoneyHelper.restar(montoEnCC, pago.getMonto());
		if (pago.getCheque() != null) {
			this.tieneChequeAsociado++;
		}
	}
	public void removePago(PagoDeClienteInmutable pago) {
		this.pagos.remove(pago);
		montoEnCC = MoneyHelper.sumar(montoEnCC, pago.getMonto());
		if (pago.getCheque() != null) {
			this.tieneChequeAsociado--;
		}
	}
	public VentaAClienteInmutable getVentaInmutable() {
		return new VentaAClienteInmutable(this.getFecha(), this.getFactura(), this.getPagos(), this.getMontoEnCC(), this.getCliente());
	}
	
	public String toString() {
		return String.format("Venta a cliente: %s %s", fecha, factura.getMonto());
	}
}
