package org.ziggurat.fenix.common.modelo.clientes;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.Set;
import java.util.TreeSet;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.ziggurat.fenix.common.modelo.comprobantes.factura.FacturaInmutable;
import org.ziggurat.fenix.common.util.MoneyHelper;

@Entity
@Table(name = "venta_a_cliente")
public class VentaAClienteInmutable extends TransaccionConCliente implements Serializable {

	/*@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;*/
	
	@Temporal(TemporalType.DATE)
	private Date fecha;
	@OneToOne(cascade = CascadeType.ALL)
    @JoinColumn(name="factura_FK")
	private FacturaInmutable factura;
	@OneToMany(fetch=FetchType.LAZY, cascade = { CascadeType.PERSIST, CascadeType.MERGE })
	private Set<PagoDeClienteInmutable> pagos = new TreeSet<PagoDeClienteInmutable>();
	private BigDecimal montoEnCC;
	private boolean tieneChequeAsociado = false;	
	
	protected VentaAClienteInmutable(Date fecha, FacturaInmutable factura,
			Set<PagoDeClienteInmutable> pagos, BigDecimal montoEnCC,
			Cliente cliente) {
		super();
		if (fecha == null) throw new NullPointerException("La fecha no puede ser null");
		if (factura == null) throw new NullPointerException("La factura no puede ser null");
		if (montoEnCC == null) throw new NullPointerException("El monto en cuenta corriente no puede ser null (puede ser 0)");
		if (cliente == null) throw new NullPointerException("El cliente no puede ser null");
		this.fecha = fecha;
		this.factura = factura;
		this.pagos = pagos;
		this.montoEnCC = montoEnCC;
		setCliente(cliente);
		BigDecimal montoEnCCEsperado = MoneyHelper.restar(factura.getMonto(), getSumaDePagos());
		BigDecimal sumaDePagos = getSumaDePagos();
		if (!MoneyHelper.equals(montoEnCCEsperado, sumaDePagos)) {
			if (montoEnCCEsperado.compareTo(montoEnCC) == 1 ) {
				throw new IllegalStateException("El monto que queda en cuenta corriente es mayor que lo que resta de factura");
			}
		}
	}
	
	private BigDecimal getSumaDePagos() {
		BigDecimal suma = BigDecimal.ZERO;
		for (PagoDeClienteInmutable pago : pagos) {
			suma = MoneyHelper.sumar(suma, pago.getMonto());
		}
		return suma;
	}
	
	public Date getFecha() {
		return fecha;
	}

	public FacturaInmutable getFactura() {
		return factura;
	}

	public Set<PagoDeClienteInmutable> getPagos() {
		return Collections.unmodifiableSet(pagos);
	}

	public BigDecimal getMontoEnCC() {
		return montoEnCC;
	}

	public boolean isTieneChequeAsociado() {
		return tieneChequeAsociado;
	}

	@Override
	public BigDecimal getMonto() {
		return factura.getMonto();
	}

	/*
	@Override
	protected void asentarMovimiento() {			
		this.getCliente().restarSaldo(this.getMonto());
		this.getCliente().addTransaccion(this);
		for (PagoDeClienteInmutable pago : this.getPagos()) {
			pago.asentarMovimiento();
		}
	}*/
	
	public boolean tieneMontoEnCuentaCorriente() {
		throw new IllegalStateException("Falta implementar");
	}

	@Override
	protected BigDecimal resolverSaldo(BigDecimal saldo) {
		return MoneyHelper.restar(saldo, this.getMonto());
	}
	
	public String toString() {
		return String.format("Venta a cliente: %s %s", fecha, factura.getMonto());
	}
}
