package org.ziggurat.fenix.common.modelo.comprobantes.factura;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.util.MoneyHelper;


public class Factura implements Serializable {
	
	private Cliente cliente;
	private BigDecimal monto = BigDecimal.ZERO;
	private Date fecha = new Date();
	private String numero;
	private TipoFactura tipoFactura;
	private List<LineaDeFacturaProducto> lineasDeFactura = new ArrayList<LineaDeFacturaProducto>();
	private List<LineaDeFacturaDescuento> lineasDeDescuento = new ArrayList<LineaDeFacturaDescuento>();
	
	public Factura() {
		
	}
	
	public Factura(Cliente cliente, Date fecha, TipoFactura tipoFactura, String numero) {
		this.cliente = cliente;
		this.fecha = fecha;
		this.tipoFactura = tipoFactura;
		this.numero = numero;
	}
	
	public Factura(Date fecha, String numero) {
		this.fecha = fecha;
		this.numero = numero;
	}
	
	public BigDecimal getMonto() {
		return monto;
	}
	
	public Date getFecha() {
		return fecha;
	}
	
	public String getNumero() {
		return numero;
	}
	
	public TipoFactura getTipoFactura() {
		return tipoFactura;
	}
	
	public void addLineaDeFactura(LineaDeFacturaProducto linea) {
		monto = MoneyHelper.sumar(monto, linea.getMontoBrutoConDescuento());
		lineasDeFactura.add(linea);
	}
	
	public void removeLineaDeFacturaProducto(LineaDeFacturaProducto linea) {
		monto = MoneyHelper.restar(monto, linea.getMontoBrutoConDescuento());
		lineasDeFactura.remove(linea);
	}
	
	public void addDescuento(LineaDeFacturaDescuento descuento) {
		monto = MoneyHelper.restar(monto, descuento.getMontoDescuento());
		lineasDeDescuento.add(descuento);
	}
	
	public void removeDescuento(LineaDeFacturaDescuento descuento) {
		monto = MoneyHelper.sumar(monto, descuento.getMontoDescuento());
		lineasDeDescuento.remove(descuento);
	}
	
	public List<LineaDeFacturaProducto> getLineasDeFactura() {
		return Collections.unmodifiableList(lineasDeFactura);
	}
	
	public List<LineaDeFacturaDescuento> getLineasDeDescuento() {
		return Collections.unmodifiableList(lineasDeDescuento);
	}
	
	public FacturaInmutable getFacturaInmutable() {		
		return new FacturaInmutable(this.cliente, this.fecha, this.monto, this.numero, this.tipoFactura, this.getListaDeLineasDeProductoInmutable(), this.getLineasDeDescuento());
	}
	
	private List<LineaDeFacturaProductoInmutable> getListaDeLineasDeProductoInmutable() {
		List<LineaDeFacturaProductoInmutable> lista = new ArrayList<LineaDeFacturaProductoInmutable>();
		for (LineaDeFacturaProducto linea : this.getLineasDeFactura()) {
			lista.add(linea.getLineaDeFacturaProductoInmutable());
		}
		return lista;
	}
	
	public BigDecimal getMontoIVA() {
		BigDecimal suma = BigDecimal.ZERO;
		for (LineaDeFacturaProducto linea : lineasDeFactura) {
			suma = MoneyHelper.sumar(linea.getMontoIVA(), suma);
		}
		return suma;
	}
	
	public BigDecimal getMontoDescuento() {
		BigDecimal suma = BigDecimal.ZERO;
		for (LineaDeFacturaDescuento linea : lineasDeDescuento) {
			suma = MoneyHelper.sumar(suma, linea.getMontoDescuento());
		}
		return suma;
	}

	public Cliente getCliente() {
		return cliente;
	}

	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
		this.tipoFactura = cliente.getResponsabilidad().getTipoFactura();
	}

	public void setMonto(BigDecimal monto) {
		this.monto = monto;
	}

	public void setFecha(Date fecha) {
		this.fecha = fecha;
	}

	public void setNumero(String numero) {
		this.numero = numero;
	}

	public void setTipoFactura(TipoFactura tipoFactura) {
		this.tipoFactura = tipoFactura;
	}

	public void setLineasDeFactura(List<LineaDeFacturaProducto> lineasDeFactura) {
		this.lineasDeFactura = lineasDeFactura;
	}

	public void setLineasDeDescuento(List<LineaDeFacturaDescuento> lineasDeDescuento) {
		this.lineasDeDescuento = lineasDeDescuento;
	}
	
}
