package org.ziggurat.fenix.common.modelo.comprobantes.factura;

import java.io.Serializable;
import java.math.BigDecimal;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "linea_de_descuento")
public class LineaDeFacturaDescuento implements ILineaDeFactura, Serializable {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	
	private BigDecimal montoDescuento = BigDecimal.ZERO;
	private String descripcion;
	
	public LineaDeFacturaDescuento(BigDecimal montoDescuento, String descripcion) {
		if (montoDescuento == null) throw new NullPointerException("El monto de descuento no puede ser null" + LineaDeFacturaDescuento.class);
		if (montoDescuento.compareTo(BigDecimal.ZERO) <= 0) {
			throw new IllegalArgumentException("El monto del descuento no puede ser negativo" + LineaDeFacturaDescuento.class);
		}
		this.montoDescuento = montoDescuento;
		this.descripcion = descripcion;
	}
	
	public BigDecimal getMontoDescuento() {
		return montoDescuento;
	}
	
	public String getDescripcion() {
		return descripcion;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof LineaDeFacturaDescuento))
			return false;
		LineaDeFacturaDescuento linea = (LineaDeFacturaDescuento) obj;
		if (!this.getMontoDescuento().equals(linea.getMontoDescuento())) return false;
		if (!this.getDescripcion().equals(linea.getDescripcion())) return false;
		return true;
	}

	@Override
	public int hashCode() {
		int result = 37;
		result = 37 * result + this.getMontoDescuento().hashCode();
		result = 37 * result + this.getDescripcion().hashCode();
		return result;
	}
}
