package org.ziggurat.fenix.common.modelo.comprobantes.factura;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;

import org.ziggurat.fenix.common.modelo.productos.Producto;
import org.ziggurat.fenix.common.util.MoneyHelper;

public class LineaDeFacturaProducto implements Serializable {

	private int cantidad = 0;
	private int porcentajeDescuento;
	private Producto producto;
	
	public LineaDeFacturaProducto(Producto producto, int cantidad, int porcentajeDescuento) {
		this.cantidad = cantidad;
		this.porcentajeDescuento = porcentajeDescuento;
		this.producto = producto;
	}
	
	
	public int getCantidad() {
		return cantidad;
	}
	
	public int getPorcentajeDescuento() {
		return porcentajeDescuento;
	}
	
	public Producto getProducto() {
		return producto;
	}
	
	/**
	 * Retorna el monto de venta del producto multiplicado por la cantidad
	 */
	public BigDecimal getMontoBrutoSinDescuento() {
		return MoneyHelper.multiplicar(producto.getPrecioVenta(), new BigDecimal(this.cantidad));
	}
	
	/**
	 * Retorna el monto de venta del producto multiplicado por la cantidad
	 * menos el descuento aplicado al item
	 */
	public BigDecimal getMontoBrutoConDescuento() {
		return MoneyHelper.restar(getMontoBrutoSinDescuento(), getMontoDescuento());
	}
	
	public BigDecimal getMontoUnitario() {
		return producto.getPrecioVenta();
	}

	public BigDecimal getMontoGravado() {
		return MoneyHelper.multiplicar(producto.getGravado(), new BigDecimal(this.cantidad));
	}

	public BigDecimal getMontoIVA() {
		return MoneyHelper.multiplicar(producto.getMontoIVA(), new BigDecimal(this.getCantidad()));
	}
	
	public BigDecimal getMontoNeto() {
		return getMontoBrutoSinDescuento().subtract(getMontoGravado().add(getMontoIVA()).setScale(3,RoundingMode.HALF_EVEN)).setScale(3,RoundingMode.HALF_EVEN);
	}
	
	public BigDecimal getMontoDescuento() {
		return getMontoBrutoSinDescuento().multiply(
				new BigDecimal(getPorcentajeDescuento()).divide(new BigDecimal("100"))
				.setScale(3,RoundingMode.HALF_EVEN)).setScale(3,RoundingMode.HALF_EVEN);
	}
	
	public void addUnidad() {
		cantidad++;
	}
	
	public void removeUnidad() {
		cantidad--;
	}
	
	public void setPorcentajeDescuento(int descuento) {
		this.porcentajeDescuento = descuento;
	}
	
	public LineaDeFacturaProductoInmutable getLineaDeFacturaProductoInmutable() {
		return new LineaDeFacturaProductoInmutable(this.producto.getProductoInmutable(),cantidad, porcentajeDescuento);
	}

	@Override
	public boolean equals(Object obj) {
		// TODO Auto-generated method stub
		return super.equals(obj);
	}

	@Override
	public int hashCode() {
		// TODO Auto-generated method stub
		return super.hashCode();
	}

	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return super.toString();
	}
		
}
