package org.ziggurat.fenix.common.modelo.comprobantes.factura;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.ziggurat.fenix.common.modelo.productos.ProductoInmutable;

@Entity
@Table(name = "linea_de_factura")
public class LineaDeFacturaProductoInmutable implements ILineaDeFactura, Serializable {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	
	private int cantidad;
	private int porcentajeDescuento;
	
	@OneToOne(cascade = CascadeType.ALL)
    @JoinColumn(name="producto_inmutable_fk")
	private ProductoInmutable producto;
	
	public LineaDeFacturaProductoInmutable(ProductoInmutable producto, int cantidad, int porcentajeDescuento) {
		this.cantidad = cantidad;
		this.producto = producto;
		this.porcentajeDescuento = porcentajeDescuento;
	}

	public int getCantidad() {
		return cantidad;
	}

	public int getPorcentajeDescuento() {
		return porcentajeDescuento;
	}

	public ProductoInmutable getProducto() {
		return producto;
	}
	
	/**
	 * Retorna el monto de venta del producto multiplicado por la cantidad
	 */
	public BigDecimal getMontoBrutoSinDescuento() {
		return producto.getPrecioVenta().multiply(new BigDecimal(this.cantidad)).setScale(3,RoundingMode.HALF_EVEN);
	}
	
	/**
	 * Retorna el monto de venta del producto multiplicado por la cantidad
	 * menos el descuento aplicado al item
	 */
	public BigDecimal getMontoBrutoConDescuento() {
		return getMontoBrutoSinDescuento().subtract(getMontoDescuento()).setScale(3,RoundingMode.HALF_EVEN);
	}
	
	public BigDecimal getMontoUnitario() {
		return producto.getPrecioVenta();
	}

	public BigDecimal getMontoGravado() {
		return producto.getGravado().multiply(new BigDecimal(this.cantidad)).setScale(3,RoundingMode.HALF_EVEN);
	}

	public BigDecimal getMontoIVA() {
		return producto.getMontoIVA().multiply(new BigDecimal(this.cantidad)).setScale(3,RoundingMode.HALF_EVEN);
	}
	
	public BigDecimal getMontoNeto() {
		return getMontoBrutoSinDescuento().subtract(getMontoGravado().add(getMontoIVA()).setScale(3,RoundingMode.HALF_EVEN)).setScale(3,RoundingMode.HALF_EVEN);
	}
	
	public BigDecimal getMontoDescuento() {
		return getMontoBrutoSinDescuento().multiply(
				new BigDecimal(getPorcentajeDescuento()).divide(new BigDecimal("100"))
				.setScale(3,RoundingMode.HALF_EVEN)).setScale(3,RoundingMode.HALF_EVEN);
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof LineaDeFacturaProductoInmutable))
			return false;
		LineaDeFacturaProductoInmutable linea = (LineaDeFacturaProductoInmutable) obj;
		if (getCantidad() != linea.getCantidad()) return false;
		if (getPorcentajeDescuento() != linea.getPorcentajeDescuento()) return false;
		if (!getProducto().equals(linea.getProducto())) return false;
		return true;
	}

	@Override
	public int hashCode() {
		int result = 37;
		result = 37 * result + this.getCantidad();
		result = 37 * result + this.getPorcentajeDescuento();
		result = 37 * result + this.getProducto().hashCode();
		return result;
	}

	@Override
	public String toString() {
		return String.format("Linea de producto inmutable: /n/t Porducto: %s /n/t Cantidad: %d /n/t Descuento: %d", this.getProducto(), this.getCantidad(), this.getPorcentajeDescuento());
	}	
	
}
