package org.ziggurat.fenix.common.modelo.impresorafiscal;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.logging.Logger;

import org.apache.commons.lang.StringUtils;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.modelo.clientes.Responsabilidad;
import org.ziggurat.fenix.common.modelo.clientes.TiposDocumentos;
import org.ziggurat.fenix.common.modelo.clientes.VentaAClienteInmutable;
import org.ziggurat.fenix.common.modelo.comprobantes.factura.LineaDeFacturaProductoInmutable;

public class ImpresoraFiscal implements ImpresoraFiscalInterface {

	private static Logger logger = Logger.getLogger("modelo.impresorafiscal.ImpresoraFiscal");
	private String puerto;
	private static String FS = "\u001c";
	private static HashMap<Responsabilidad, String> responsabilidades = new HashMap<Responsabilidad, String>();
	private static HashMap<TiposDocumentos, String> tiposDocumento = new HashMap<TiposDocumentos, String>();
	private static String[] statusImpresora = new String[16];
	private static String[] statusFiscal = new String[16];
	
	static {
		responsabilidades.put(Responsabilidad.RESPONSABLE_INSCRIPTO, "I");
		responsabilidades.put(Responsabilidad.RESPONSABLE_NO_INSCRIPTO, "N");
		responsabilidades.put(Responsabilidad.EXENTO, "E");
		responsabilidades.put(Responsabilidad.NO_RESPONSABLE, "A");
		responsabilidades.put(Responsabilidad.VENTA_BIENES_DE_USO, "B");
		responsabilidades.put(Responsabilidad.RESPONSABLE_MONOTRIBUTO, "M");
		responsabilidades.put(Responsabilidad.CONSUMIDOR_FINAL, "C");
		
		tiposDocumento.put(TiposDocumentos.CUIT, "C");
		tiposDocumento.put(TiposDocumentos.LE, "0");
		tiposDocumento.put(TiposDocumentos.LC, "1");
		tiposDocumento.put(TiposDocumentos.DNI, "2");
		tiposDocumento.put(TiposDocumentos.CI, "4");
		
		statusImpresora[0] = "";
		statusImpresora[1] = "Cajon de dinero cerrado o ausente";
		statusImpresora[2] = "";
		statusImpresora[3] = "";
		statusImpresora[4] = "";
		statusImpresora[5] = "";
		statusImpresora[6] = "";
		statusImpresora[7] = "Tapa de impresora abierta";
		statusImpresora[8] = "Buffer de impresora vacio";
		statusImpresora[9] = "Buffer de impresora lleno";
		statusImpresora[10] = "Falta papel de tickets";
		statusImpresora[11] = "Falta papel del diario";
		statusImpresora[12] = "Impresora offline";
		statusImpresora[13] = "Error de impresora";
		statusImpresora[14] = "";
		statusImpresora[15] = "Impresora ocupada";
		
		statusFiscal[0] = "";
		statusFiscal[1] = "";
		statusFiscal[2] = "Documento abierto";
		statusFiscal[3] = "Documento fiscal abierto";
		statusFiscal[4] = "Error en ingreso de fecha";
		statusFiscal[5] = "Terminal fiscal certificada";
		statusFiscal[6] = "Terminal fiscal certificada";
		statusFiscal[7] = "Memoria fiscal a punto de llenarse";
		statusFiscal[8] = "Memoria fiscal llena, desbordada o dada de baja";
		statusFiscal[9] = "Desborde del total";
		statusFiscal[10] = "Comando no valido para el estado fiscal actual";
		statusFiscal[11] = "Datos no validos en un campo";
		statusFiscal[12] = "Comando desconocido";
		statusFiscal[13] = "";
		statusFiscal[14] = "Error de chequeo de memoria de trabajo";
		statusFiscal[15] = "Error en chequeo de memoria fiscal";
	}
	
	protected ImpresoraFiscal(String puerto) {
		if (StringUtils.isNotBlank(puerto)) {
			this.puerto = puerto;
		} else {
			puerto = "1";
		}
		logger.info("Creando impresora fiscal en puerto " + puerto);
	}
	
	private String toASCIIString(String string) {
		if (StringUtils.isNotBlank(string)) {
			byte[] asciiBytes = {0};
			try {
				asciiBytes = string.getBytes("US-ASCII");
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return new String(asciiBytes);
		} else {
			return StringUtils.EMPTY;
		}
	}
	
	private String obtenerStatusFiscal(final String status) {
		char[] statusArray = status.toCharArray();
		StringBuffer respuesta = new StringBuffer();
		for (int i = 0; i < 16; i++) {
			if (statusArray[i] == '1') {
				respuesta.append(statusFiscal[i] + " ");
			}
		}
		return respuesta.toString();
	}
	
	private String obtenerStatusImpresora(final String status) {
		char[] statusArray = status.toCharArray();
		StringBuffer respuesta = new StringBuffer();
		for (int i = 0; i < 16; i++) {
			if (statusArray[i] == '1') {
				respuesta.append(statusImpresora[i] + " ");
			}
		}
		return respuesta.toString();
	}
	
	private String[] runCommand(String command) throws ImpresoraFiscalExcepcion {
		logger.info("Ejecutando comando en spooler: " + command);
		StringBuilder str2 = new StringBuilder();
		try {
			StringBuilder str = new StringBuilder();
			String line;
			Process p;
			if (System.getProperty("os.name").toUpperCase().indexOf("LINUX") == -1) {
				p = Runtime.getRuntime().exec("spooler/win32/wspooler.exe -p" + puerto + " -l -n -c \"" + command + "\"");
			} else {
				p = Runtime.getRuntime().exec("spooler/linux/spooler -ptty" + puerto + " -l -n -c \"" + command + "\"");
			}
			
			BufferedReader input = new BufferedReader(new InputStreamReader(p.getInputStream()));
			while ((line = input.readLine()) != null) {				
				str.append(line);
				str.append("\n");
			}
			input.close();
			if (StringUtils.isNotBlank(str.toString())) {
				throw new ImpresoraFiscalExcepcion(str.toString());
			}
			BufferedReader in = new BufferedReader(new FileReader("respuesta.ans"));
	        String parsedString;
	        while ((parsedString = in.readLine()) != null) {
	        	str2.append(parsedString);
	        }
	        in.close();	        	        	        	        
		} catch (Exception err) {
			err.printStackTrace();
		}		
		return StringUtils.split(str2.toString(), "|");
	}
	
	/**
	 * Abre una factura y retorna el numero
	 * de la factura abierta
	 * @return El numero de factura abierta
	 * @throws ImpresoraFiscalExcepcion 
	 */
	private String abrirFactura(Cliente cliente) throws ImpresoraFiscalExcepcion {
		StringBuilder command = new StringBuilder();
		command.append("@");
		command.append(FS);
		if ((cliente.getResponsabilidad().equals(Responsabilidad.RESPONSABLE_INSCRIPTO)) || (cliente.getResponsabilidad().equals(Responsabilidad.RESPONSABLE_NO_INSCRIPTO)))
			command.append("A");
		else
			command.append("B");
		command.append(FS);
		command.append("S");		
		String[] response = runCommand(command.toString());
		return response[3];
	}
	
	private void setDatosCliente(Cliente cliente) throws ImpresoraFiscalExcepcion {
		StringBuilder command = new StringBuilder();
		command.append("b");
		command.append(FS);
		command.append(toASCIIString(StringUtils.substring(cliente.getNombre(), 0, 49)));
		command.append(FS);
		command.append(toASCIIString(cliente.getDocumento()));
		command.append(FS);
		command.append(responsabilidades.get(cliente.getResponsabilidad()));
		command.append(FS);
		command.append(tiposDocumento.get(cliente.getTipo_doc()));
		command.append(FS);
		command.append(toASCIIString(cliente.getDomicilio())); //TODO Verificar el limite de caracteres de este campo
		runCommand(command.toString());
	}
	
	private void imprimirLinea(LineaDeFacturaProductoInmutable linea) throws ImpresoraFiscalExcepcion {
		StringBuilder command = new StringBuilder();
		command.append("B");
		command.append(FS);
		command.append(toASCIIString(StringUtils.substring(linea.getProducto().getDescripcion() + " - " + linea.getProducto().getMarca(), 0, 49)));
		command.append(FS);
		command.append(String.valueOf(linea.getCantidad()) + ".0" );
		command.append(FS);
		command.append(linea.getMontoUnitario());
		command.append(FS);
		command.append(linea.getProducto().getPorcentajeIVA().toString() + ".0");
		command.append(FS);
		command.append("M"); //TODO Verificar que este valor sea el correcto !!!
		command.append(FS);
		command.append("0.0");
		command.append(FS);
		command.append("0");
		command.append(FS);
		command.append("T"); //TODO Verificar que este valor sea el correcto !!!
		
		runCommand(command.toString());
	}
	
	private void subTotal() throws ImpresoraFiscalExcepcion {
		StringBuilder command = new StringBuilder();
		command.append("C");
		command.append(FS);
		command.append("P");
		command.append(FS);
		command.append("L"); // Aca puede ir un caracter cualquiera de acuerdo a la documentacion
		command.append(FS);
		command.append("0");				
		String[] response = runCommand(command.toString());
	}
	
	
	/**
	 * Calcula el total de la factura y retorna el vuelto o el monto faltante
	 * (por la manera en que esta implementado esta clase el vuelto deberia ser
	 * 0 en todos los casos
	 */
	private String total(VentaAClienteInmutable venta) throws ImpresoraFiscalExcepcion {
		StringBuilder command = new StringBuilder();
		command.append("D");
		command.append(FS);
		if (!venta.tieneMontoEnCuentaCorriente())
			command.append("EFECTIVO");
		else
			command.append("CUENTA CORRIENTE");
		command.append(FS);
		command.append(venta.getMonto());
		command.append(FS);
		command.append("T");
		command.append(FS);
		command.append("0");
		
		String[] response = runCommand(command.toString());
		return response[3];
	}
	
	/**
	 * Cierra el comprobante fiscal abierto
	 * y retorna el numero del comprobante cerrado.
	 * @return El numero de comprobante cerrado
	 */
	private String cerrarComprobanteFiscal() throws ImpresoraFiscalExcepcion {
		String[] response = runCommand("E");
		return response[3];
	}
	
	/**
	 * Pide un status a la impresora y retorna
	 * los valores de dicho status.
	 * @return
	 * @throws ImpresoraFiscalExcepcion 
	 * @throws ImpresoraFiscalExcepcion 
	 */
	synchronized public StatusImpresora getStatus() throws ImpresoraFiscalExcepcion {
		String[] response = runCommand("*");
		StatusImpresora status = new StatusImpresora();
		status.setUltimoNumeroComprobanteA(response[5]);
		status.setUltimoNumeroComprobanteBC(response[3]);
		status.setUltimoNumeroNotaDeCreditoA(response[8]);
		status.setUltimoNumeroNotaDeCreditoBC(response[7]);
		status.setUltimoNumeroRemito(response[9]);
		return status;
	}
	
	synchronized public String getProximoNumeroDeFacturaA() throws ImpresoraFiscalExcepcion {
		StatusImpresora status = this.getStatus();
		int numero = Integer.decode(status.getUltimoNumeroComprobanteA());
		return String.valueOf(numero + 1);
	}
	
	synchronized public String getProximoNumeroDeFacturaBC() throws ImpresoraFiscalExcepcion {
		StatusImpresora status = this.getStatus();
		int numero = Integer.decode(status.getUltimoNumeroComprobanteBC());
		return String.valueOf(numero + 1);
	}
	
	/*
	public RespuestaDeImpresoraFiscal cierreDiarioZ() {
		return runCommand("9" + FS + "Z");
	}
	
	private RespuestaDeImpresoraFiscal sendComandoIncorrecto() {
		return runCommand("13");
	}*/
	
	private void cancelarComando() throws ImpresoraFiscalExcepcion {
		String comando = Character.valueOf((char)255).toString();
		runCommand(comando);
	}
	
	private void cancelarTodo() throws ImpresoraFiscalExcepcion {
		try {
			cancelarComando();
		} catch (ImpresoraFiscalExcepcion imp) {
		}
		try {
			cerrarComprobanteFiscal();
		} catch (ImpresoraFiscalExcepcion imp) {
		}
		
	}
	
	synchronized public void imprimirTransaccion(VentaAClienteInmutable venta) throws ImpresoraFiscalExcepcion {
		try {
			setDatosCliente(venta.getCliente());
			abrirFactura(venta.getCliente());
			for (LineaDeFacturaProductoInmutable linea : venta.getFactura().getLineasDeFactura()) {
				imprimirLinea(linea);
			}
			subTotal();
			total(venta);
			cerrarComprobanteFiscal();
		} catch (ImpresoraFiscalExcepcion e) {
			e.printStackTrace();
			cancelarTodo();
			throw e;
		}
	}
}
