package org.ziggurat.fenix.common.modelo.matafuego;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.ziggurat.fenix.common.modelo.clientes.Cliente;


/**
 * Esta clase representa a Matafuego. Contiene el nombre del proprietario
 * asi como la direccion. Tambien se puede realizar la asosiacion entre
 * el aparato con un cliente. En principio, si el matafuegos tiene un cliente
 * asociado, el nombre y la direccion del matafuegos deberian ser iguales
 * que las del cliente.
 * 
 * @author manuel
 *
 */


@Entity
@Table(name="matafuegos")
public class Matafuego implements Serializable {
	
	@Id @GeneratedValue(strategy=GenerationType.AUTO)
	private long id;
	
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	private String nombre;
	private String direccion;
	@Column(unique=true)
	private String nroAparato;
	private String marca;
	private String idInterna;
	private String ubicacion;
	private double capacidad;
	private int anoFabricacion;
	
	@Temporal(TemporalType.DATE) 
	private Date proximaPruebaHidraulica;
	@Temporal(TemporalType.DATE)
	private Date proximoMantenimiento;
	
	public Date getProximaPruebaHidraulica() {
		return proximaPruebaHidraulica;
	}

	public void setProximaPruebaHidraulica(Date proximaPruebaHidraulica) {
		this.proximaPruebaHidraulica = proximaPruebaHidraulica;
	}

	public Date getProximoMantenimiento() {
		return proximoMantenimiento;
	}

	public void setProximoMantenimiento(Date proximoMantenimiento) {
		this.proximoMantenimiento = proximoMantenimiento;
	}

	@Enumerated(EnumType.STRING)
	private TipoMatafuego tipoMatafuego;
	
	@ManyToOne
	private Cliente cliente = null;
	
	public Matafuego() {
		
	}
	
	public Matafuego(String nroAparato) {
		if (nroAparato == null) throw new NullPointerException("El numero de aparato en matafuego no puede ser null");
		this.setNroAparato(nroAparato);
	}

	public String getNombre() {
		return nombre;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public String getDireccion() {
		return direccion;
	}

	public void setDireccion(String direccion) {
		this.direccion = direccion;
	}

	public String getNroAparato() {
		return nroAparato;
	}

	public void setNroAparato(String nroAparato) {
		this.nroAparato = nroAparato;
	}

	public String getMarca() {
		return marca;
	}

	public void setMarca(String marca) {
		this.marca = marca;
	}

	public String getIdInterna() {
		return idInterna;
	}

	public void setIdInterna(String idInterna) {
		this.idInterna = idInterna;
	}

	public String getUbicacion() {
		return ubicacion;
	}

	public void setUbicacion(String ubicacion) {
		this.ubicacion = ubicacion;
	}

	public double getCapacidad() {
		return capacidad;
	}

	public void setCapacidad(double capacidad) {
		this.capacidad = capacidad;
	}

	public int getAnoFabricacion() {
		return anoFabricacion;
	}

	public void setAnoFabricacion(int anoFabricacion) {
		this.anoFabricacion = anoFabricacion;
	}

	public TipoMatafuego getTipoMatafuego() {
		return tipoMatafuego;
	}

	public void setTipoMatafuego(TipoMatafuego tipoMatafuego) {
		this.tipoMatafuego = tipoMatafuego;
	}

	public Cliente getCliente() {
		return cliente;
	}

	public void setCliente(Cliente cliente) {
		this.cliente = cliente;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + anoFabricacion;
		long temp;
		temp = Double.doubleToLongBits(capacidad);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result
				+ ((nroAparato == null) ? 0 : nroAparato.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Matafuego other = (Matafuego) obj;
		if (anoFabricacion != other.anoFabricacion)
			return false;
		if (Double.doubleToLongBits(capacidad) != Double
				.doubleToLongBits(other.capacidad))
			return false;
		if (nroAparato == null) {
			if (other.nroAparato != null)
				return false;
		} else if (!nroAparato.equals(other.nroAparato))
			return false;
		return true;
	}
	
	public String toString() {
		return String.format("[Matafuego: %s \n\t -marca: %s \n\t -capacidad %s]", this.getNroAparato(), this.getMarca(), this.getCapacidad());
	}

}
