package org.ziggurat.fenix.common.modelo.productos;

import java.io.Serializable;
import java.math.BigDecimal;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

import org.ziggurat.fenix.common.util.FenixMathContext;

@Entity
@Table(name = "producto")
public class Producto implements Serializable {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	
	private String marca;
	private String descripcion;
	private BigDecimal precioCosto = BigDecimal.ZERO; // Es el precio al que le compran al mayorista.
	private BigDecimal porcentajeIVA = new BigDecimal("21"); // Es el porcentaje del iva	
    private BigDecimal gravado = BigDecimal.ZERO;; // Es el porcentaje de impuestos gravados de este producto
    private BigDecimal precioVenta = BigDecimal.ZERO;; // Es el precio de venta = ((precioCosto + ganacia) + IVA) + gravado
    @Enumerated(EnumType.STRING)
	private Rubro rubro;
	
	public Producto() {
		
	}

	public String getMarca() {
		return marca;
	}

	public void setMarca(String marca) {
		this.marca = marca;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public void setDescripcion(String descripcion) {
		this.descripcion = descripcion;
	}

	public BigDecimal getPrecioCosto() {
		return precioCosto;
	}

	public void setPrecioCosto(BigDecimal precioCosto) {
		this.precioCosto = precioCosto;
	}

	public BigDecimal getPorcentajeIVA() {
		return porcentajeIVA;
	}

	public void setPorcentajeIVA(BigDecimal porcentajeIVA) {
		this.porcentajeIVA = porcentajeIVA;
	}

	public BigDecimal getGravado() {
		return gravado;
	}

	public void setGravado(BigDecimal gravado) {
		this.gravado = gravado;
	}

	public BigDecimal getPrecioVenta() {
		return precioVenta;
	}

	public void setPrecioVenta(BigDecimal precioVenta) {
		this.precioVenta = precioVenta;
	}

	public Rubro getRubro() {
		return rubro;
	}

	public void setRubro(Rubro rubro) {
		this.rubro = rubro;
	}

	/**
	 * El monto del IVA es el porcentaje aplicado al precio de costo + ganancia
	 * @return
	 */
	public BigDecimal getMontoIVA() {
		return getPrecioVenta().multiply(getPorcentajeIVA().divide(new BigDecimal("100"),FenixMathContext.CTX),FenixMathContext.CTX);		
	}
	
	public ProductoInmutable getProductoInmutable() {
		return new ProductoInmutable(this);
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Producto))
			return false;
		Producto prod = (Producto) obj;
		if (!this.getMarca().equals(prod.getMarca())) return false;
		if (!this.getDescripcion().equals(prod.getDescripcion())) return false;
		return true;
	}

	@Override
	public int hashCode() {
		int result = 37;
		result = 37 * result + this.getMarca().hashCode();
		result = 37 * result + this.getDescripcion().hashCode();
		return result;
	}

	@Override
	public String toString() {		
		return String.format("Producto: %s, %s", this.getMarca(), this.getDescripcion()); 
	}

}
