package org.ziggurat.fenix.common.modelo.productos;

import java.io.Serializable;
import java.math.BigDecimal;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

import org.ziggurat.fenix.common.util.FenixMathContext;


@Entity
@Table(name = "producto_inmutable")
public class ProductoInmutable implements Serializable {
	
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	
	private String marca;
	private String descripcion;
	private BigDecimal precioCosto; // Es el precio al que le compran al mayorista.
	private BigDecimal porcentajeIVA; // Es el porcentaje del iva	
	private BigDecimal gravado; // Es el porcentaje de impuestos gravados de este producto
	private BigDecimal precioVenta; // Es el precio de venta = ((precioCosto + ganacia) + IVA) + gravado
	@Enumerated(EnumType.STRING)
	private Rubro rubro;
	
	public ProductoInmutable(Producto producto) {
		this.marca = producto.getMarca();
		this.descripcion = producto.getDescripcion();
		this.precioCosto = producto.getPrecioCosto();
		this.precioVenta = producto.getPrecioVenta();
		this.gravado = producto.getGravado();
		this.porcentajeIVA = producto.getPorcentajeIVA();
		this.rubro = producto.getRubro();
	}

	public String getMarca() {
		return marca;
	}

	public String getDescripcion() {
		return descripcion;
	}

	public BigDecimal getPrecioCosto() {
		return precioCosto;
	}

	public BigDecimal getPorcentajeIVA() {
		return porcentajeIVA;
	}

	public BigDecimal getGravado() {
		return gravado;
	}

	public BigDecimal getPrecioVenta() {
		return precioVenta;
	}
	
	public Rubro getRubro() {
		return rubro;
	}
	
	/**
	 * El monto del IVA es el porcentaje aplicado al precio de costo + ganancia
	 * @return
	 */
	public BigDecimal getMontoIVA() {
		return getPrecioVenta().multiply(getPorcentajeIVA().divide(new BigDecimal("100"),FenixMathContext.CTX),FenixMathContext.CTX);		
	}
	
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof ProductoInmutable))
			return false;
		ProductoInmutable prod = (ProductoInmutable) obj;
		if (!this.getMarca().equals(prod.getMarca())) return false;
		if (!this.getDescripcion().equals(prod.getDescripcion())) return false;
		return true;
	}

	@Override
	public int hashCode() {
		int result = 37;
		result = 37 * result + this.getMarca().hashCode();
		result = 37 * result + this.getDescripcion().hashCode();
		return result;
	}

	@Override
	public String toString() {		
		return String.format("ProductoInmutable (Marca: %s, Descripcion: %s, PrecioVenta: %s)", this.getMarca(), this.getDescripcion(), this.getPrecioVenta().toPlainString()); 
	}
}
