package org.ziggurat.fenix.common.modelo.proveedores;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.ziggurat.fenix.common.modelo.transacciones.TipoDeTransaccion;
import org.ziggurat.fenix.common.util.MoneyHelper;

public class CCProveedor implements Serializable {

	private BigDecimal saldoPrevio = BigDecimal.ZERO;
	private List<LineaCCProveedor> lineasDeCC = new ArrayList<LineaCCProveedor>();
	//private BigDecimal saldoActual = BigDecimal.ZERO;
	private BigDecimal saldoAcumulado = BigDecimal.ZERO;
	private Date desde;
	private Date hasta;
	
	/**
	 * Crea un resumen de la cuenta corriente del proveedor.
	 * Para armar el resumen, se utiliza la lista de transacciones recibidas como parametro
	 * y el saldoInicial se utiliza como punto de partida para realizar los calculos de
	 * los montos parciales. 
	 * @param desde
	 * @param hasta
	 * @param transacciones
	 * @param saldoInicial
	 */
	public CCProveedor(List<TransaccionConProveedor> transacciones, BigDecimal saldoInicial) {
		if (transacciones == null) throw new NullPointerException("En CCProveedor, la lista de transacciones no puede ser null");
		if (transacciones.size() > 0) {
			this.desde = transacciones.get(0).getFecha();
			this.hasta = transacciones.get(transacciones.size() - 1).getFecha();
		}
		this.saldoPrevio = saldoInicial;
		this.saldoAcumulado = saldoInicial;
		for (TransaccionConProveedor tr : transacciones) {
			addTransaccion(tr);
		}
	}
	/*
	public CCProveedor(List<TransaccionConProveedor> transacciones, BigDecimal saldoInicial) {
		this.saldoPrevio = saldoInicial;
		this.saldoAcumulado = saldoInicial;
		for (TransaccionConProveedor tr : transacciones) {
			addTransaccion(tr);
		}
	}
	*/
	
	private void addTransaccion(TransaccionConProveedor tr) {
		LineaCCProveedor nuevaLinea = null;		
		if (tr.getTipoDeTransaccion().equals(TipoDeTransaccion.COMPRA_A_PROVEEDOR)) {
			CompraAProveedorInmutable compra = (CompraAProveedorInmutable) tr;
			saldoAcumulado = MoneyHelper.restar(saldoAcumulado, compra.getMonto());
			nuevaLinea = new LineaCCProveedor(compra.getNotas(), compra.getMonto(), compra.getFecha(), null, compra.getNumero(), saldoAcumulado, "Factura", tr);
		} else if (tr.getTipoDeTransaccion().equals(TipoDeTransaccion.PAGO_A_PROVEEDOR_CHEQUE)) {
			PagoAProveedorInmutable pago = (PagoAProveedorInmutable)tr;
			saldoAcumulado = MoneyHelper.sumar(saldoAcumulado, pago.getMonto());
			nuevaLinea = new LineaCCProveedor(pago.getNotas(), null, pago.getFecha(), pago.getMonto(), pago.getNroRecibo(), saldoAcumulado, "Pago con cheque", pago);
		} else if (tr.getTipoDeTransaccion().equals(TipoDeTransaccion.PAGO_A_PROVEEDOR_CONTADO)) {
			PagoAProveedorInmutable pago = (PagoAProveedorInmutable)tr;
			saldoAcumulado = MoneyHelper.sumar(saldoAcumulado, pago.getMonto());
			nuevaLinea = new LineaCCProveedor(pago.getNotas(), null, pago.getFecha(), pago.getMonto(), pago.getNroRecibo(), saldoAcumulado, "Pago efectivo", pago);
		}		
		lineasDeCC.add(nuevaLinea);
	}

	public BigDecimal getSaldoPrevio() {
		return saldoPrevio;
	}

	public List<LineaCCProveedor> getLineasDeCC() {
		return lineasDeCC;
	}

	public BigDecimal getSaldoActual() {
		return saldoAcumulado;
	}

	public Date getDesde() {
		return desde;
	}

	public Date getHasta() {
		return hasta;
	}
	
}
