package org.ziggurat.fenix.common.modelo.proveedores;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.ziggurat.fenix.common.modelo.transacciones.TipoDeTransaccion;
import org.ziggurat.fenix.common.util.MoneyHelper;



@Entity
@Table(name = "compras_a_proveedor")
public class CompraAProveedorInmutable extends TransaccionConProveedor implements Serializable {

	@Temporal(TemporalType.DATE)
	private Date fecha;
	private String numero;
	private BigDecimal netoSinIVA;
	private BigDecimal montoIVA;
	private BigDecimal noGravadoExento;
	private BigDecimal percepciones;
	@Column(nullable=false, scale=3)
	private BigDecimal total;
	
	protected CompraAProveedorInmutable() {}
	
	public CompraAProveedorInmutable(final Date fecha, Proveedor proveedor, String numero, BigDecimal netoSinIVA,
			BigDecimal montoIVA, BigDecimal noGravadoExento,
			BigDecimal percepciones, BigDecimal total) {
		if (null == fecha) throw new NullPointerException("La fecha es null");
		if (null == proveedor) throw new  NullPointerException("El proveedor es null");
		if (proveedor == null) throw new NullPointerException("El proveedor no puede ser null");
		if (fecha == null) throw new NullPointerException("La fecha no puede ser null");
		if (netoSinIVA == null) throw new NullPointerException("El neto sin iva no puede ser null");
		if (montoIVA == null) throw new NullPointerException("El monto del iva no puede ser null");
		if (noGravadoExento == null) throw new NullPointerException("El monto no gravado no puede ser null");
		if (percepciones == null) throw new NullPointerException("Las percepciones no pueden ser null");
		if (total == null) throw new NullPointerException("El total no puede ser null");
		this.fecha = fecha;
		//this.proveedor = proveedor;
		setProveedor(proveedor);
		this.fecha = fecha;
		this.numero = numero;
		this.netoSinIVA = netoSinIVA;
		this.montoIVA = montoIVA;
		this.noGravadoExento = noGravadoExento;
		this.percepciones = percepciones;
		this.total = total;
		BigDecimal suma = MoneyHelper.sumar(netoSinIVA, MoneyHelper.sumar(montoIVA, MoneyHelper.sumar(noGravadoExento, percepciones)));
		if (!MoneyHelper.equals(suma, total)) {
			throw new IllegalStateException("El valor total no se corresponde a la suma de las partes");
		}
	}
	
	@Override
	public Date getFecha() {
		return fecha;
	}

	@Override
	public BigDecimal getMonto() {
		return getTotal();
	}
	
	public String toString() {
		return String.format("Compra a proveedor: [%s] [%s]", getFecha(), getMonto());
	}

	@Override
	public TipoDeTransaccion getTipoDeTransaccion() {
		return TipoDeTransaccion.COMPRA_A_PROVEEDOR;
	}
	
	@Override
	protected BigDecimal resolverSaldo(BigDecimal saldo) {
		return MoneyHelper.restar(saldo, this.getMonto());		
	}
	
	public String getNumero() {
		return numero;
	}

	public BigDecimal getNetoSinIVA() {
		return netoSinIVA;
	}

	public BigDecimal getMontoIVA() {
		return montoIVA;
	}

	public BigDecimal getNoGravadoExento() {
		return noGravadoExento;
	}

	public BigDecimal getPercepciones() {
		return percepciones;
	}

	public BigDecimal getTotal() {
		return total;
	}
	
}
