package org.ziggurat.fenix.common.modelo.proveedores;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.TipoDeTransaccion;
import org.ziggurat.fenix.common.util.MoneyHelper;

@Entity
@Table(name = "pagos_a_proveedor")
public class PagoAProveedorInmutable extends TransaccionConProveedor implements Serializable {

	@Temporal(TemporalType.DATE)
	private Date fecha;
	@OneToOne(cascade = CascadeType.ALL)
	@JoinColumn(name = "cheque_FK")
	private Cheque cheque;
	@Column(name="monto",nullable=false,scale=3)
	private BigDecimal monto;
	@Column(name="nroRecibo")
	private String nroRecibo;

	protected PagoAProveedorInmutable() {
		
	}
	
	public PagoAProveedorInmutable(Proveedor proveedor, Date fecha, Cheque cheque, String notas, String nroRecibo) {
		super();
		this.fecha = fecha;
		this.cheque = cheque;
		setProveedor(proveedor);
		//this.notas = notas;
		setNotas(notas);
		this.monto = cheque.getMonto();
		this.nroRecibo = nroRecibo;
	}

	public PagoAProveedorInmutable(Proveedor proveedor, Date fecha, String notas, BigDecimal monto, String nroRecibo) {
		super();
		this.fecha = fecha;
		setProveedor(proveedor);
		setNotas(notas);		
		this.monto = monto;
		this.nroRecibo = nroRecibo;
	}

	public Cheque getCheque() {
		return cheque;
	}

	public String getNroRecibo() {
		return nroRecibo;
	}

	@Override
	public Date getFecha() {
		return fecha;
	}

	@Override
	public BigDecimal getMonto() {
		return monto;
	}
	
	public String toString() {
		return String.format("Pago a proveedor: [%s] [%s]", getFecha(), getMonto());
	}

	@Override
	public TipoDeTransaccion getTipoDeTransaccion() {
		if (this.getCheque() == null) {
			return TipoDeTransaccion.PAGO_A_PROVEEDOR_CONTADO;
		} else {
			return TipoDeTransaccion.PAGO_A_PROVEEDOR_CHEQUE;
		}		
	}
	
	@Override
	protected BigDecimal resolverSaldo(BigDecimal saldo) {
		return MoneyHelper.sumar(saldo, this.getMonto());		
	}

}
