package org.ziggurat.fenix.common.modelo.proveedores;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

import org.ziggurat.fenix.common.util.MoneyHelper;

public class ResumenMensual implements Serializable {

	private List<ResumenPorTipo> resumenPorTipo;
	private BigDecimal totalPagos = BigDecimal.ZERO;

	private BigDecimal netoSinIVA = BigDecimal.ZERO;
	private BigDecimal montoIVA = BigDecimal.ZERO;
	private BigDecimal noGravadoExento = BigDecimal.ZERO;
	private BigDecimal percepciones = BigDecimal.ZERO;
	private BigDecimal totalCompras = BigDecimal.ZERO;

	public ResumenMensual(List<ResumenPorTipo> resumenes) {
		this.resumenPorTipo = resumenes;
		for (ResumenPorTipo resumen : resumenes) {
			netoSinIVA = MoneyHelper.sumar(netoSinIVA, resumen.getNetoSinIVA());
			montoIVA = MoneyHelper.sumar(montoIVA, resumen.getMontoIVA());
			noGravadoExento = MoneyHelper.sumar(noGravadoExento, resumen.getNoGravadoExento());
			percepciones = MoneyHelper.sumar(percepciones, resumen.getPercepciones());
			totalCompras = MoneyHelper.sumar(totalCompras, resumen.getTotalFacturas());
			totalPagos = MoneyHelper.sumar(totalPagos, resumen.getTotalPagos());
		}
	}

	public List<ResumenPorTipo> getResumenPorTipo() {
		return resumenPorTipo;
	}

	public BigDecimal getTotalPagos() {
		return totalPagos;
	}

	public BigDecimal getNetoSinIVA() {
		return netoSinIVA;
	}

	public BigDecimal getMontoIVA() {
		return montoIVA;
	}

	public BigDecimal getNoGravadoExento() {
		return noGravadoExento;
	}

	public BigDecimal getPercepciones() {
		return percepciones;
	}

	public BigDecimal getTotalCompras() {
		return totalCompras;
	}
	
	/**
	 * Diferencia entre pagos y compras
	 * Pagos - compras
	 * @return
	 */
	public BigDecimal getSaldo() {
		return MoneyHelper.restar(getTotalPagos(), getTotalCompras());
	}
	
	public int getAno() {
		if (resumenPorTipo.size() > 0)
			return resumenPorTipo.get(0).getAno();
		return 0;
	}
	
	public int getMes() {
		if (resumenPorTipo.size() > 0)
			return resumenPorTipo.get(0).getMes();
		return 0;
	}
	
}
