package org.ziggurat.fenix.common.modelo.proveedores;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.ziggurat.fenix.common.util.MoneyHelper;

public class ResumenPorTipo implements Serializable {

	private CategoriaDeProveedor tipo;
	private int mes;
	private int ano;
	
	private BigDecimal totalPagos = BigDecimal.ZERO;
	
	private BigDecimal netoSinIVA = BigDecimal.ZERO;
	private BigDecimal montoIVA = BigDecimal.ZERO;
	private BigDecimal noGravadoExento = BigDecimal.ZERO;
	private BigDecimal percepciones = BigDecimal.ZERO;
	private BigDecimal totalFacturas = BigDecimal.ZERO;
	
	private List<CompraAProveedorInmutable> compras;
	private List<PagoAProveedorInmutable> pagos;
	
	public ResumenPorTipo(CategoriaDeProveedor tipo, int ano, int mes, List<CompraAProveedorInmutable> compras, List<PagoAProveedorInmutable> pagos) {		
		this.tipo = tipo;
		this.compras = compras;
		this.pagos = pagos;
		this.mes = mes;
		this.ano = ano;
		for (CompraAProveedorInmutable compra : compras) {
			netoSinIVA = MoneyHelper.sumar(netoSinIVA, compra.getNetoSinIVA());
			montoIVA = MoneyHelper.sumar(montoIVA, compra.getMontoIVA());
			noGravadoExento = MoneyHelper.sumar(noGravadoExento, compra.getNoGravadoExento());
			percepciones = MoneyHelper.sumar(percepciones, compra.getPercepciones());
			totalFacturas = MoneyHelper.sumar(totalFacturas, compra.getMonto());
		}

		for (PagoAProveedorInmutable pago : pagos) {
			totalPagos = MoneyHelper.sumar(totalPagos, pago.getMonto());
		}
	}

	public CategoriaDeProveedor getTipo() {
		return tipo;
	}

	public int getMes() {
		return mes;
	}

	public int getAno() {
		return ano;
	}

	public BigDecimal getTotalPagos() {
		return totalPagos;
	}

	public BigDecimal getNetoSinIVA() {
		return netoSinIVA;
	}

	public BigDecimal getMontoIVA() {
		return montoIVA;
	}

	public BigDecimal getNoGravadoExento() {
		return noGravadoExento;
	}

	public BigDecimal getPercepciones() {
		return percepciones;
	}

	public BigDecimal getTotalFacturas() {
		return totalFacturas;
	}

	public List<CompraAProveedorInmutable> getCompras() {
		return Collections.unmodifiableList(compras);
	}

	public List<PagoAProveedorInmutable> getPagos() {
		return Collections.unmodifiableList(pagos);
	}
	
	/**
	 * Retorna la diferencia entre los pagos
	 * y las facturas (pagos - facturas)
	 * @return
	 */
	public BigDecimal getSaldo() {
		return MoneyHelper.restar(getTotalPagos(), getTotalFacturas());
	}
	
	public List<TransaccionConProveedor> getTodasTransaccionesOrdenadas() {
		List<TransaccionConProveedor> listaCompleta = new ArrayList<TransaccionConProveedor>(compras);
		listaCompleta.addAll(pagos);
		Collections.sort(listaCompleta);
		return listaCompleta;
	}
	
}
