package org.ziggurat.fenix.common.modelo.proveedores;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.ziggurat.fenix.common.modelo.transacciones.TipoDeTransaccion;

@Entity
@Inheritance(strategy=InheritanceType.TABLE_PER_CLASS)
//@Table(name = "transaccion_con_proveedor")
abstract public class TransaccionConProveedor implements Comparable<TransaccionConProveedor>, Serializable {

	
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	private long id;
	
	@ManyToOne( cascade = {CascadeType.PERSIST, CascadeType.MERGE} )
	private Proveedor proveedor;
	private String notas;
	@Column(nullable=false)
	private boolean eliminado = false;
	
	@Temporal(TemporalType.DATE)
	private Date fechaAltaRegistro = new Date();
	
	public boolean isEliminado() {
		return eliminado;
	}
	public void setEliminado(boolean eliminado) {
		this.eliminado = eliminado;
	}
	abstract public Date getFecha();
	abstract public BigDecimal getMonto();
	//abstract protected void asentarMovimiento();
	abstract public TipoDeTransaccion getTipoDeTransaccion();
	//abstract public boolean isVigente();
	/**
	 * Este metodo es un metodo utilitario para ayudar a calcular
	 * saldos parciales. Se le pasa un saldo como parametro
	 * y dependiendo del tipo de movimiento, se retorna
	 * el valor del nuevo saldo con el movimiento aplicado.
	 * @param saldo
	 * @return
	 */
	abstract protected BigDecimal resolverSaldo(BigDecimal saldo);

	
	
	@Override
	public int compareTo(TransaccionConProveedor o) {
		if (getFecha().compareTo(o.getFecha()) == 0) {
			if (this.getTipoDeTransaccion().equals(TipoDeTransaccion.COMPRA_A_PROVEEDOR)) {
				return -1;
			} else {
				return 1;
			}
		} else {
			return getFecha().compareTo(o.getFecha());
		}
		
	}
	public Proveedor getProveedor() {
		return proveedor;
	}
	protected void setProveedor(Proveedor proveedor) {
		this.proveedor = proveedor;
	}
	public String getNotas() {
		return notas;
	}
	public void setNotas(String notas) {
		this.notas = notas;
	}
	
}
