package org.ziggurat.fenix.common.services.matafuegos;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.time.DateUtils;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.exception.ConstraintViolationException;
import org.ziggurat.fenix.common.database.hibernate.GenericHibernateRepository;
import org.ziggurat.fenix.common.database.hibernate.HibernateUtil;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.modelo.matafuego.Matafuego;
import org.ziggurat.fenix.common.modelo.matafuego.ServicioMatafuego;

class HibMatafuegosRepository extends GenericHibernateRepository<Matafuego, Long> {

	@Override
	public Matafuego makePersistent(Matafuego entity) {
		try {
			return super.makePersistent(entity);
		} catch(ConstraintViolationException cve) {
			entity.setId(0);
			throw cve;
		}
	}

	@Override
	public List<Matafuego> findAll() {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Matafuego.class)
			.list();
	}
	
	
	/**
	 * Use findAllPaginated of the superclass instead
	 * @param start
	 * @param count
	 * @return
	 */
	@Deprecated
	public List<Matafuego> findMatafuegosPage(int start, int count) {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Matafuego.class)
			.setFirstResult(start)
			.setMaxResults(count)
			.list();
	}
	
	public int countMatafuegos() {
		Session session = HibernateUtil.currentSession();
		return (Integer) session.createCriteria(Matafuego.class)
			.setProjection(Projections.rowCount())
			.uniqueResult();
	}
	
	public List<ServicioMatafuego> getServicios(Matafuego matafuego) {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(ServicioMatafuego.class)
			.add(Restrictions.eq("matafuego", matafuego))
			.addOrder(Order.desc("fecha"))
			.list();
	}
	
	public ServicioMatafuego getUltimoServicio(Matafuego matafuego) {
		Session session = HibernateUtil.currentSession();
		return (ServicioMatafuego) session.createCriteria(ServicioMatafuego.class)
			.add(Restrictions.eq("matafuego", matafuego))
			.setProjection(Projections.max("fecha"))
			.uniqueResult();
	}
	
	public List<ServicioMatafuego> getPruebasHidraulicas(Matafuego matafuego) {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(ServicioMatafuego.class)
			.add(Restrictions.eq("matafuego", matafuego))
			.add(Restrictions.eq("pruebaHidraulica", true))
			.addOrder(Order.desc("fecha"))
			.list();
	}
	
	public List<Matafuego> getMatafuegosDeCliente(Cliente cliente) {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Matafuego.class)
			.add(Restrictions.eq("cliente", cliente))
			.list();
	}

    public List<Matafuego> getMatafuegosParaRenovarDps() {
        Session session = HibernateUtil.currentSession();
		return session.createCriteria(ServicioMatafuego.class)
            .add(Restrictions.between("proximoMantenimiento",DateUtils.add(new Date(), Calendar.MONTH, -2), DateUtils.add(new Date(), Calendar.MONTH, 1)))
			.addOrder(Order.desc("proximoMantenimiento"))
			.list();
    }
	
}
