package org.ziggurat.fenix.common.services.matafuegos;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.NullArgumentException;
import org.apache.commons.lang.time.DateUtils;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.modelo.matafuego.Matafuego;
import org.ziggurat.fenix.common.modelo.matafuego.ServicioMatafuego;
import org.ziggurat.fenix.common.services.FenixServiceException;

public class MatafuegosService {

	HibMatafuegosRepository repoMatafuegos = new HibMatafuegosRepository();
	HibServicioMatafuegoRepository repoServicioMatafuego = new HibServicioMatafuegoRepository();
	
	
	/**
	 * 
	 * @return la cantidad de matafuegos que hay en el sistema
	 */
	public int countMatafuegos() {
		return repoMatafuegos.countMatafuegos();
	}
	
	/**
	 * Retorna una lista de matafuegos "paginada"
	 * @param start
	 * @param count
	 * @return Lista de matafuegos
	 */
	public List<Matafuego> findAllPaginated(int start, int count) {
		return repoMatafuegos.findAllPaginated(start, count);
	}
	
	/**
	 * Busca matafuegos utilizando una instancia de ejemplo
	 * @param exampleInstance
	 * @param excludeProperty
	 * @return
	 */
	public List<Matafuego> findByExample(Matafuego exampleInstance,
			String... excludeProperty) {
		return repoMatafuegos.findByExample(exampleInstance, excludeProperty);
	}
	
	@Deprecated
	public List<Matafuego> findMatafuegosPage(int start, int count) {
		return repoMatafuegos.findMatafuegosPage(start, count);
	}
	
	/**
	 * 
	 * @param cliente
	 * @return Lista de matafuegos de un cliente
	 */
	public List<Matafuego> getMatafuegosDeCliente(Cliente cliente) {
		return repoMatafuegos.getMatafuegosDeCliente(cliente);
	}
	
	public Matafuego save(Matafuego entity) {
		return repoMatafuegos.makePersistent(entity);
	}
	
	public void makeTransient(Matafuego entity) {
		repoMatafuegos.makeTransient(entity);
	}
	
	public List<ServicioMatafuego> getServiciosDelMatafuego(Matafuego matafuego) {
		return repoMatafuegos.getServicios(matafuego);
	}
	
	public List<ServicioMatafuego> getPruebasHidrahulicasDelMatafuego(Matafuego matafuego) {
		return repoMatafuegos.getPruebasHidraulicas(matafuego);
	}
	
	public ServicioMatafuego getUltimoServicioDelMatafuego(Matafuego matafuego) {
		return repoMatafuegos.getUltimoServicio(matafuego);
	}
	
	public void saveServicioMatafuego(ServicioMatafuego servicio) throws FenixServiceException {
		if (servicio.getMatafuego() == null) throw new NullArgumentException("matafuego no puede ser null");
		Matafuego matafuego = servicio.getMatafuego();
		ServicioMatafuego ultimoServicio = getUltimoServicioDelMatafuego(matafuego);
		if (ultimoServicio.getFecha().after(servicio.getFecha())) throw new FenixServiceException("La fecha de es anterior al ultimo servicio realizado");
		Date proximoMantenimiento = DateUtils.add(servicio.getFecha(), Calendar.YEAR, 1);
		matafuego.setProximoMantenimiento(proximoMantenimiento);
		if (servicio.isPruebaHidraulica()) {
			Date proximaPH = DateUtils.add(servicio.getFecha(), Calendar.YEAR, matafuego.getTipoMatafuego().getDuracionPH());
			matafuego.setProximaPruebaHidraulica(proximaPH);
		}
		repoServicioMatafuego.makePersistent(servicio);
		repoMatafuegos.makePersistent(matafuego);
	}
}
