package org.ziggurat.fenix.common.util;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.time.DateUtils;
import org.databene.benerator.Generator;
import org.databene.benerator.factory.GeneratorFactory;
import org.databene.model.function.Sequence;
import org.ziggurat.fenix.common.database.hibernate.HibProveedorRepository;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.modelo.clientes.Responsabilidad;
import org.ziggurat.fenix.common.modelo.clientes.TiposDocumentos;
import org.ziggurat.fenix.common.modelo.matafuego.Matafuego;
import org.ziggurat.fenix.common.modelo.proveedores.CompraAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.PagoAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;

public class RandomDataGenerator {

	static private Generator<String> nombres = GeneratorFactory.getSampleGenerator(
			"Adrián","Cipriano","Ezequiel","Hernán","Kaleb","Oziel","Agustín",
			"Delmiro","Fabricio","Husai","Kirios","Pablo","Alan","David","Fantino",
			"Huapi","Lautaro","Pío","Baltasar","Diego","Fausto","Iago","Ludovico",
			"Román","Bautista","Dylan","Federico","Ian","Magno","Rubén","Benjamín",
			"Edelmiro","Gabino","Ismael","Martiniano","Santino","Benicio","Elias","Gadiel",
			"Jair","Nazareno","Tadeo","Camilo","Elian","Galo","Jaro","Nereo","Uziel","Conrado",
			"Emiliano","Gregorio","Justino","Octavio","Valentín");
	
	static private Generator<String> apellidos = GeneratorFactory.getSampleGenerator(
			"Alarma","Alonso","Celada","Alloza","Amírola","Andrés","Antu","Ariño",
			"Arroyabe","Artal","Ayete","Bagorro","Belenguer","Bellmunt","Bernaldo","Quirós",
			"Bonastre","Candil","Canto","Caracuel","Castellanos","Cembellín","Clopés",
			"Coscino","D'Elia","Ducazcal","Fuentelzas","GilaArmas","Monsalve","Gortázar",
			"Haro","Holgado","Lardiés","Lecubarri","Machargo","Maspoch","Mateo","Millarengo",
			"Miquel","Molano","Monsalvo","Montañés","Mortal","Murcia","Orjuela","Orsolich",
			"Ossa","Panizo","Passano","Peyroteo","Portugal","Portal","Portol","Quaintenne",
			"Quecedo","Quílez","Rozalén","Rufián","Salgado","Sanou","Sistiaga");
	
	//static private Generator<TipoDeProveedor> tiposDeProveedor = GeneratorFactory.getSampleGenerator(TipoDeProveedor.values());
	
	static private Generator<String> cuits = GeneratorFactory.getSampleGenerator(
			"30702434110","30702435036","30702435109","30702436571","30702436954",
			"30702439929","30702441486","30702444698","30702445996","30707025286",
			"30707033998","30707042563","30707042598","30707043349","30707043551",
			"30707045961","30707058737","30707060553","30707068570","30707070613",
			"30707087125","30707088881","30707152083","30707162550","30707164545",
			"30707171185","30707178392","30707188789","30707211675","30707235825",
			"30707237607","30707239081","30707244883","30707247319","30707260773",
			"30707284192","30707284842","30707291121","30707312234","30707314237",
			"30707320091","30707345442","30707356703","30707357440","30707359923",
			"30707360387","30707360522","30707364935","30707380612","30707394281",
			"30707398333","30707406255","30707428402","30707437517","30707450807",
			"30707456287","30707464174","30707487301","30707496491","30707496661",
			"30707500464","30707503188","30707526021","30707539034","30707550011",
			"30707562222","30707563334","30707569987","30707571663","30707578722",
			"30707583459","30707584773","30707636773","30707640169","30707684247",
			"30707706917","30707711449","30707714146","30707718850","30707724168",
			"30707745483","30707747893","30707756124","30707757996","30707767940",
			"30707780246","30707783520","30707832181","30707853553","30707856862",
			"30707859772","30707866531","30707866620","30707876448","30707877819",
			"30707880003","30707898948","30707901159","30707902910","30707903100",
			"30707904530","30707945342","30707947388","30707949747","30707953949",
			"30707956514","30707959122","30707971726","30707982132","30707990933",
			"30708007001","30708030607","30708044551","30708046716","30708054298",
			"30708065060","30708075236","30708103167","30708114185","30708123656");
	
	static private Generator<String> bancos = GeneratorFactory.getSampleGenerator(
			"Banco Rio","Standard Bank","HSBC","Galicia","Banco Frances");

	static private Generator<String> marcas = GeneratorFactory.getSampleGenerator(
			"Boris", "OMBU", "GRAFA", "PAMPERO", "ARCOTEX", "CARAN", "FADESA", "DRAGO"
	);
	
	//private final String BIGDECIMAL_PATTERN = "[1-9]{2,3}\\.[0-9]{2,2}";
	static private final Generator<BigDecimal> bigDecimalGenerator = GeneratorFactory.getNumberGenerator(BigDecimal.class, BigDecimal.ZERO, new BigDecimal("999.99"), new BigDecimal("0.11"), Sequence.RANDOM, 0);
	
	static private final Generator<Integer> yearGenerator = GeneratorFactory.getNumberGenerator(Integer.class, new Integer("1990"), new Integer("2008"), new Integer("1"), Sequence.RANDOM, 0);
	
	static private final String[] parse = {"dd/MM/yyyy"};
//	private final Date startDate = DateUtils.parseDate("01/01/2004", parse);
	static private final Generator<Date> dateGenerator = GeneratorFactory.getDateGenerator(DateUtils.add(Calendar.getInstance().getTime(), Calendar.YEAR, -2), Calendar.getInstance().getTime(), 1, Sequence.RANDOM, 0);
	
	static private final Generator<Integer> intsGenerator = GeneratorFactory.getNumberGenerator(Integer.class, new Integer("0"), new Integer("9999999"), new Integer("1"), Sequence.RANDOM, 0);

	public RandomDataGenerator() throws Exception {
	}
	
	public void generateProveedores() {
		
		HibProveedorRepository repo = new HibProveedorRepository();		
		for (int i = 0; i < 150; i++) {
			Proveedor p = new Proveedor();
			p.setNombre(nombres.generate() + " " + apellidos.generate());
			//p.setTipoDeGasto(tiposDeProveedor.generate());
			p.setCUIT(cuits.generate());
			repo.makePersistent(p);
		}
		nombres.close();
		apellidos.close();
	}
	
	public void generateTransacciones() {
		HibProveedorRepository repo = new HibProveedorRepository();
		HibTransaccionConProveedorRepository trRepo = new HibTransaccionConProveedorRepository();
		List<Proveedor> listaProveedores = repo.findAll();
		for (Proveedor p : listaProveedores) {
			for (int i=0;i < 20; i++) {
				PagoAProveedorInmutable pago = 
					new PagoAProveedorInmutable(p, dateGenerator.generate(), "Notas", bigDecimalGenerator.generate(), intsGenerator.generate().toString());
					trRepo.makePersistent(pago);
			}
			for (int i=0; i < 20; i++) {
				BigDecimal m1 = bigDecimalGenerator.generate();
				BigDecimal m2 = bigDecimalGenerator.generate();
				BigDecimal m3 = bigDecimalGenerator.generate();
				BigDecimal m4 = bigDecimalGenerator.generate();
				
				BigDecimal total = MoneyHelper.sumar(m1, MoneyHelper.sumar(m2, MoneyHelper.sumar(m3, m4)));
				Date fecha = dateGenerator.generate(); 
				
				//FacturaDeProveedor factura = new FacturaDeProveedor(p, fecha, intsGenerator.generate().toString(), m1, m2, m3, m4, total);
				CompraAProveedorInmutable compra = new CompraAProveedorInmutable(fecha, p, intsGenerator.generate().toString(), m1, m2, m3, m4, total);
				trRepo.makePersistent(compra);
			}
			//repo.makePersistent(p);
		}
	}
	
	static public PagoAProveedorInmutable getRandomPagoEfectivoFromProveedor(Proveedor p) {
		PagoAProveedorInmutable pago = 
			new PagoAProveedorInmutable(p, dateGenerator.generate(), "Notas", bigDecimalGenerator.generate(), intsGenerator.generate().toString());
		return pago;
	}
	
	static public PagoAProveedorInmutable getRandomPagoEfectivoFromProveedor(Proveedor p, Date fechaInicial, Date fechaFinal) {
		Generator<Date> dateGenerator = GeneratorFactory.getDateGenerator(fechaInicial, fechaFinal, 1, Sequence.RANDOM, 0);
		PagoAProveedorInmutable pago = 
			new PagoAProveedorInmutable(p, dateGenerator.generate(), "Notas", bigDecimalGenerator.generate(), intsGenerator.generate().toString());
		return pago;
	}
	
	static public PagoAProveedorInmutable getRandomPagoConChequeToProveedor(Proveedor p) {
		Cheque cheque = new Cheque();
		cheque.setALaOrdenDe(nombres.generate() + " " + apellidos.generate());
		cheque.setBanco("LALALA");
		cheque.setCUIT("234234234234");
		cheque.setDescripcion("asdasd");
		cheque.setEntregadoPor("asdasd");
		Date fecha = dateGenerator.generate();
		cheque.setFecha(fecha);
		cheque.setFechaDeCobro(dateGenerator.generate());
		cheque.setNumeroCheque("123123");
		BigDecimal monto = bigDecimalGenerator.generate(); 
		cheque.setMonto(monto);
		
		PagoAProveedorInmutable pago = 
			new PagoAProveedorInmutable(p, dateGenerator.generate(), cheque, "Notas", intsGenerator.generate().toString());
		return pago;
	}
	
	static public CompraAProveedorInmutable getRandomCompraAProveedor(Proveedor p) {
		BigDecimal m1 = bigDecimalGenerator.generate();
		BigDecimal m2 = bigDecimalGenerator.generate();
		BigDecimal m3 = bigDecimalGenerator.generate();
		BigDecimal m4 = bigDecimalGenerator.generate();
		
		BigDecimal total = MoneyHelper.sumar(m1, MoneyHelper.sumar(m2, MoneyHelper.sumar(m3, m4)));
		CompraAProveedorInmutable compra = new CompraAProveedorInmutable(
				dateGenerator.generate(),
				p,
				intsGenerator.generate().toString(),
				m1, m2, m3, m4,
				total				
				);
		return compra;
	}
	
	static public CompraAProveedorInmutable getRandomCompraAProveedor(Proveedor p, int mes, int ano) {
		String[] parse = { "yyyy-MM-dd" };
		Date fechaInicio = null;
		try {
			fechaInicio = DateUtils.parseDate(ano + "-" + mes + "-01", parse);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Date fechaFin = DateUtils.addDays(DateUtils.addMonths(fechaInicio, 1), -1);
		return getRandomCompraAProveedor(p, fechaInicio, fechaFin);
	}
	
	static public CompraAProveedorInmutable getRandomCompraAProveedor(Proveedor p, Date fechaInicial, Date fechaFinal) {
		Generator<Date> dateGenerator = GeneratorFactory.getDateGenerator(fechaInicial, fechaFinal, 1, Sequence.RANDOM, 0);
		BigDecimal m1 = bigDecimalGenerator.generate();
		BigDecimal m2 = bigDecimalGenerator.generate();
		BigDecimal m3 = bigDecimalGenerator.generate();
		BigDecimal m4 = bigDecimalGenerator.generate();
		
		BigDecimal total = MoneyHelper.sumar(m1, MoneyHelper.sumar(m2, MoneyHelper.sumar(m3, m4)));
		CompraAProveedorInmutable compra = new CompraAProveedorInmutable(
				dateGenerator.generate(),
				p,
				intsGenerator.generate().toString(),
				m1, m2, m3, m4,
				total				
				);
		return compra;
	}
	
	static public Cheque getRandomCheque(EstadoDeCheque estado) {
		Cheque cheque = new Cheque();
		cheque.setALaOrdenDe(nombres.generate() + " " + apellidos.generate());
		cheque.setBanco(bancos.generate());
		cheque.setCUIT(cuits.generate());
		cheque.setDescripcion("asdasd");
		cheque.setEntregadoPor(nombres.generate() + " " + apellidos.generate());
		cheque.setEstado(estado);
		cheque.setFecha(dateGenerator.generate());
		cheque.setFechaDeCobro(dateGenerator.generate());
		cheque.setMonto(bigDecimalGenerator.generate());
		cheque.setNumeroCheque(String.valueOf(intsGenerator.generate()));
		return cheque;
	}
	
	static public Cliente getRandomCliente() {
		Cliente c = new Cliente();
		c.setCelular(String.valueOf(intsGenerator.generate()));
		c.setDocumento(String.valueOf(intsGenerator.generate()));
		c.setDomicilio("Un domicilio");
		c.setEmail("mail@mail.com");
		c.setFax("123123");
		c.setLocalidad("Localidad");
		c.setNombre(nombres.generate() + " " + apellidos.generate());
		c.setResponsabilidad(Responsabilidad.RESPONSABLE_INSCRIPTO);
		c.setTipo_doc(TiposDocumentos.DNI);
		return c;
	}
	
	static public Matafuego getRandomMatafuego() {
		Cliente c = getRandomCliente();		
		return getRandomMatafuegoDeCliente(c);
	}
	
	static public Matafuego getRandomMatafuegoDeCliente(Cliente c) {		
		Matafuego m = new Matafuego();
		m.setAnoFabricacion(yearGenerator.generate().intValue());
		m.setCapacidad(bigDecimalGenerator.generate().doubleValue());
		m.setCliente(c);
		m.setDireccion(c.getDomicilio());
		m.setMarca("FADESA");
		return m;
	}
	
	public void generateBigDecimalsTest() {
		for (int i=0; i< 10; i++) {
			System.out.println(dateGenerator.generate());
		}
	}
		
	public static void main(String[] args) {		
		try {
			RandomDataGenerator dataGen = new RandomDataGenerator();
			dataGen.generateProveedores();
			dataGen.generateTransacciones();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

}
