package org.ziggurat.fenix.common.database.hibernate;

import junit.framework.TestCase;

import org.ziggurat.fenix.common.database.hibernate.HibChequesRepository;
import org.ziggurat.fenix.common.database.hibernate.HibernateUtil;
import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;
import org.ziggurat.fenix.common.util.RandomDataGenerator;

public class HibChequesRepositoryTest extends TestCase {
	
	HibChequesRepository repo = new HibChequesRepository();
	
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		HibernateUtil.deleteAllTransacciones();
		HibernateUtil.deleteAllCheques();
	}
	
	public void testListarCheques() {
		Cheque c1 = RandomDataGenerator.getRandomCheque(EstadoDeCheque.EN_POSESION_DE_FENIX);
		repo.makePersistent(c1);
		HibernateUtil.closeSession();
		
		Cheque c2 = RandomDataGenerator.getRandomCheque(EstadoDeCheque.EN_POSESION_DE_FENIX);
		repo.makePersistent(c2);
		HibernateUtil.closeSession();
		
		Cheque c3 = RandomDataGenerator.getRandomCheque(EstadoDeCheque.EN_POSESION_DE_FENIX);
		repo.makePersistent(c3);
		HibernateUtil.closeSession();
		
		assertEquals(3, repo.getChequesPorEstado(EstadoDeCheque.EN_POSESION_DE_FENIX).size());
		assertEquals(0, repo.getChequesPorEstado(EstadoDeCheque.DEPOSITADO).size());
		assertEquals(0, repo.getChequesPorEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR).size());
	}
	
	public void testEliminarCheque() {
		Cheque c1 = RandomDataGenerator.getRandomCheque(EstadoDeCheque.EN_POSESION_DE_FENIX);
		repo.makePersistent(c1);
		HibernateUtil.closeSession();
		
		Cheque c2 = RandomDataGenerator.getRandomCheque(EstadoDeCheque.EN_POSESION_DE_FENIX);
		repo.makePersistent(c2);
		HibernateUtil.closeSession();
		
		Cheque c3 = RandomDataGenerator.getRandomCheque(EstadoDeCheque.EN_POSESION_DE_FENIX);
		repo.makePersistent(c3);
		HibernateUtil.closeSession();
		
		assertEquals(3, repo.getChequesPorEstado(EstadoDeCheque.EN_POSESION_DE_FENIX).size());
		assertEquals(0, repo.getChequesPorEstado(EstadoDeCheque.DEPOSITADO).size());
		assertEquals(0, repo.getChequesPorEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR).size());
		HibernateUtil.closeSession();
		
		
		repo.eliminarCheque(c1);
		HibernateUtil.closeSession();
		assertEquals(2, repo.getChequesPorEstado(EstadoDeCheque.EN_POSESION_DE_FENIX).size());
		assertEquals(0, repo.getChequesPorEstado(EstadoDeCheque.DEPOSITADO).size());
		assertEquals(0, repo.getChequesPorEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR).size());
	}
	
	

}
