package org.ziggurat.fenix.common.database.hibernate;

import java.util.List;

import junit.framework.TestCase;

import org.ziggurat.fenix.common.modelo.proveedores.CategoriaDeProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.common.services.proveedores.CategoriasDeProveedorService;

public class HibProveedorRepositoryTest extends TestCase {
	
	HibProveedorRepository repo = new HibProveedorRepository();
	HibTransaccionConProveedorRepository repoTrans = new HibTransaccionConProveedorRepository();
	
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		HibernateUtil.deleteAllTransacciones();
		HibernateUtil.deleteAllProveedores();
		HibernateUtil.deleteAllCheques();
	}
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	/**
	 * Este test se implemento para verificar un problema
	 * con un query polimorfico de hibernate (que no
	 * pudo ser solucionado)
	 */
//	public void testRecuperarTransaccion() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");		
//		repo.makePersistent(p);
//		HibernateUtil.closeSession();
//		PagoAProveedorInmutable pago = new PagoAProveedorInmutable(p, new Date(), "Notas", new BigDecimal("123.34"), "1234");
//		repoTrans.makePersistent(pago);
//		HibernateUtil.closeSession();
//		PagoAProveedorInmutable recu = (PagoAProveedorInmutable) repo.getPagosDeProveedor(p).get(0);
//		assertNotNull(recu.getMonto());
//	}
//	
	public void testPersistirProveedor() {
		CategoriasDeProveedorService catService = new CategoriasDeProveedorService();
		
		CategoriaDeProveedor cat = new CategoriaDeProveedor("LALA", "lala");
		CategoriaDeProveedor cat2 = new CategoriaDeProveedor("LALA2", "lala2");
		
		catService.saveCategoria(cat);
		catService.saveCategoria(cat2);
		
		Proveedor p = new Proveedor("Manuel", "Notas", cat, "320548");		
		repo.makePersistent(p);
		Proveedor rec = repo.getProveedoresPorNombre("Manuel").get(0);
		assertEquals("Manuel", p.getNombre());
		assertEquals("320548", p.getCUIT());
		assertEquals("Notas", p.getNotas());
		//assertEquals(BigDecimal.ZERO, p.getSaldo());
		assertEquals(cat, p.getCategoria());
		assertEquals(p.getNombre(), rec.getNombre());
		assertEquals(p.getCUIT(), rec.getCUIT());
		assertEquals(p.getNotas(), rec.getNotas());
		//assertEquals(p.getSaldo(), rec.getSaldo());
		assertEquals(p.getCategoria(), rec.getCategoria());
		assertEquals(p, rec);
		assertEquals(1, repo.findAll().size());
		assertEquals(1, repo.getProveedoresPorCategoria(cat).size());
		assertEquals(0, repo.getProveedoresPorCategoria(cat2).size());
		repo.makeTransient(p);
		assertEquals(0, repo.findAll().size());
	}
	
	public void testBuscarProveedorPorEjemplo() {
		
		CategoriasDeProveedorService catService = new CategoriasDeProveedorService();
		
		CategoriaDeProveedor cat = new CategoriaDeProveedor("LALA", "lala");
		CategoriaDeProveedor cat2 = new CategoriaDeProveedor("LALA2", "lala2");
		
		catService.saveCategoria(cat);
		catService.saveCategoria(cat2);
		
		Proveedor p = new Proveedor("Manuel", "Notas", cat, "320548");		
		repo.makePersistent(p);
		HibernateUtil.closeSession();
		
		
		Proveedor p1 = new Proveedor("Maria", "Notas", cat, "320548");		
		repo.makePersistent(p1);
		HibernateUtil.closeSession();
		
		Proveedor p2 = new Proveedor("Lucas", "Notas", cat, "320548");		
		repo.makePersistent(p2);
		HibernateUtil.closeSession();
		
		Proveedor p3 = new Proveedor("Carmen", "Notas", cat, "320548");		
		repo.makePersistent(p3);
		HibernateUtil.closeSession();
		
		Proveedor aBuscar = new Proveedor();
		aBuscar.setNombre("Manuel");
		List<Proveedor> result = repo.findByExample(aBuscar);
		assertEquals(1, result.size());
		
	}
	
//	public void testPagoDeProveedorEfectivo() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");		
//		repo.makePersistent(p);
//		PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p);
//		repoTrans.makePersistent(pago);
//		assertTrue(MoneyHelper.equals(repo.getSaldoDeProveedor(p), pago.getMonto()));
//		List<TransaccionConProveedor> lista = repo.getTransaccionesDeProveedor(p);
//		assertEquals(1, lista.size());
//		
//		repoTrans.makeTransient(pago);
//		List<TransaccionConProveedor> lista2 = repo.getTransaccionesDeProveedor(p);
//		assertEquals(0, lista2.size());
//	}
//	
//	public void testCompraAProveedor() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//	}
//	
//	public void testPagoDeProveedorCheque() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");		
//		repo.makePersistent(p);
//		PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoConChequeToProveedor(p);
//		repoTrans.makePersistent(pago);
//		assertTrue(MoneyHelper.equals(repo.getSaldoDeProveedor(p), pago.getMonto()));		
//		List<PagoAProveedorInmutable> lista = repo.getPagosDeProveedorConCheques(p);		
//		assertEquals(1, lista.size());
//		assertEquals(pago, lista.get(0));
//		repoTrans.makeTransient(pago);
//		List<TransaccionConProveedor> lista2 = repo.getTransaccionesDeProveedor(p);
//		List<PagoAProveedorInmutable> lista3 = repo.getPagosDeProveedorConCheques(p);		
//		assertEquals(1, lista.size());
//		assertEquals(0, lista2.size());
//		assertEquals(0, lista3.size());
//	}
//	
//	public void testEliminarCompraAProveedor() {
//		//assertTrue(false);
//	}
//	
//	public void testEliminarPagoEfectivo() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");		
//		repo.makePersistent(p);
//		PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p);
//		repoTrans.makePersistent(pago);
//		assertTrue(MoneyHelper.equals(repo.getSaldoDeProveedor(p), pago.getMonto()));
//		repoTrans.eliminarTransaccion(pago);
//		assertTrue(MoneyHelper.equals(repo.getSaldoDeProveedor(p), BigDecimal.ZERO));
//	}
//	
//	public void testEliminarPagoCheque() {
//		//assertTrue(false);
//	}
//	
//	public void testCuentaCorriente() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		HibernateUtil.closeSession();
//		
//		PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoConChequeToProveedor(p);
//		repoTrans.makePersistent(pago);
//		HibernateUtil.closeSession();
//		
//		PagoAProveedorInmutable pago2 = RandomDataGenerator.getRandomPagoConChequeToProveedor(p);
//		repoTrans.makePersistent(pago2);
//		HibernateUtil.closeSession();
//		
//		CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra);
//		HibernateUtil.closeSession();
//		
//		CompraAProveedorInmutable compra2 = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra2);
//		HibernateUtil.closeSession();
//		
//		CCProveedor cc = repo.getCuentaCorriente(p);
//		assertEquals(4, cc.getLineasDeCC().size());
//	}
//	
//	public void testSaldoDeProveedor() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		BigDecimal expectedSaldo = BigDecimal.ZERO;
//		
//		PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoConChequeToProveedor(p);
//		repoTrans.makePersistent(pago);
//		expectedSaldo = MoneyHelper.sumar(expectedSaldo, pago.getMonto());
//		
//		PagoAProveedorInmutable pago2 = RandomDataGenerator.getRandomPagoConChequeToProveedor(p);
//		repoTrans.makePersistent(pago2);
//		expectedSaldo = MoneyHelper.sumar(expectedSaldo, pago2.getMonto());
//		
//		CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra);
//		expectedSaldo = MoneyHelper.restar(expectedSaldo, compra.getMonto());
//		
//		CompraAProveedorInmutable compra2 = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra2);
//		expectedSaldo = MoneyHelper.restar(expectedSaldo, compra2.getMonto());
//		
//		BigDecimal saldo = repo.getSaldoDeProveedor(p);
//		assertTrue(saldo.equals(expectedSaldo));
//	}
//	
//	public void testResumenPorTipo() throws Exception {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, 5, 2008);
//			repoTrans.makePersistent(compra);
//		}
//		
//		assertTrue(repoTrans.getResumenPorTipo(5, 2008, TipoDeProveedor.MERCADERIA).getCompras().size() == 10);
//	}
//	
//	public void testLibroDeIvaCompras() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra);
//		
//		Date fecha = compra.getFecha();
//		Calendar cal = Calendar.getInstance();
//		cal.setTime(fecha);
//		int mes = cal.get(Calendar.MONTH) + 1;
//		int ano = cal.get(Calendar.YEAR);
//		
//		CompraAProveedorInmutable compra2 = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra2);
//		
//		CompraAProveedorInmutable compra3 = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra3);
//		
//		CompraAProveedorInmutable compra4 = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra4);
//		
//		LibroDeIvaCompras libro = repoTrans.getLibroDeIvaCompras(mes, ano);
//		assertTrue(libro.getFacturas().size() > 0);
//		HibernateUtil.commitTransaction();
//	}
//	/*
//	public void testGetSaldoEntreFechas() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, 5, 2008);
//			repoTrans.makePersistent(compra);
//		}
//		for (int i=0;i<5;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, 6, 2008);
//			repoTrans.makePersistent(compra);
//		}
//		for (int i=0;i<8;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, 7, 2008);
//			repoTrans.makePersistent(compra);
//		}
//		
//		Calendar cal1 = Calendar.getInstance();
//		cal1.set(Calendar.DAY_OF_MONTH, 1);
//		cal1.set(Calendar.MONTH, 4);
//		cal1.set(Calendar.YEAR, 2008);
//		Date fin = DateUtils.add(DateUtils.add(cal1.getTime(), Calendar.MONTH, 1), Calendar.DAY_OF_MONTH, -1);
//		
//		List<TransaccionConProveedor> transac = repoTrans.getTransaccionesBetweenDates(cal1.getTime(), fin);
//		assertEquals(10, transac.size());
//		
//		cal1 = Calendar.getInstance();
//		cal1.set(Calendar.DAY_OF_MONTH, 1);
//		cal1.set(Calendar.MONTH, 5);
//		cal1.set(Calendar.YEAR, 2008);
//		fin = DateUtils.add(DateUtils.add(cal1.getTime(), Calendar.MONTH, 1), Calendar.DAY_OF_MONTH, -1);
//		
//		List<TransaccionConProveedor> transac2 = repoTrans.getTransaccionesBetweenDates(cal1.getTime(), fin);
//		assertEquals(5, transac2.size());
//		
//		List<CompraAProveedorInmutable> transac3 = repoTrans.getComprasBetweenDates(cal1.getTime(), fin);
//		assertEquals(5, transac3.size());
//		
//	}*/
//	
//	public void testGastoDelMes() throws Exception {
//		BigDecimal saldo = BigDecimal.ZERO;
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, 5, 2008);
//			repoTrans.makePersistent(compra);
//			saldo = MoneyHelper.sumar(saldo, compra.getMonto());
//		}
//		assertTrue(repoTrans.getGastoDelMes(5, 2008).equals(saldo));
//		
//	}
//	
//	public void testContarProveedores() {
//		Proveedor p = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		Proveedor p2 = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p2);
//		Proveedor p3 = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p3);
//		assertEquals(3, repo.countProveedores());
//	}
//	
//	public void testContarProveedoresFiltrados() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		Proveedor p2 = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p2);
//		Proveedor p3 = new Proveedor("Jose", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p3);
//		assertEquals(1, repo.countProveedoresFiltrados("maria"));
//		assertEquals(1, repo.countProveedoresFiltrados("manuel"));
//		assertEquals(1, repo.countProveedoresFiltrados("jose"));
//		assertEquals(2, repo.countProveedoresFiltrados("ma"));
//		assertEquals(2, repo.countProveedoresFiltrados("e"));
//	}
//	
//	public void testPaginaDeListaDeProveedores() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		Proveedor p2 = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p2);
//		Proveedor p3 = new Proveedor("Jose", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p3);
//		assertEquals(repo.findListaDeProveedoresPage(0, 2).size(), 2);
//		assertEquals(repo.findListaDeProveedoresPage(0, 2).get(0).getNombre(), "Jose");
//		assertEquals(repo.findListaDeProveedoresPage(0, 2).get(1).getNombre(), "Manuel");
//	}
//	
//	public void testPaginaDeListaDeProveedoresFiltrada() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		Proveedor p2 = new Proveedor("Manuel", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p2);
//		Proveedor p3 = new Proveedor("Jose", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p3);
//		assertEquals(repo.findListaDeProveedoresPage(0, 2, "e").size(), 2);
//		assertEquals(repo.findListaDeProveedoresPage(0, 2, "maria").size(), 1);
//		assertEquals(repo.findListaDeProveedoresPage(0, 2, "e").get(0).getNombre(), "Jose");
//		assertEquals(repo.findListaDeProveedoresPage(0, 2, "e").get(1).getNombre(), "Manuel");
//	}
//	
//	public void testSaldoDeProveedorHasta() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		Calendar cal = Calendar.getInstance();
//		cal.set(Calendar.DAY_OF_MONTH, 4);
//		cal.set(Calendar.MONTH, 4);
//		cal.set(Calendar.YEAR, 2008);
//		Date desde = cal.getTime();
//		Date hasta = DateUtils.add(desde, Calendar.MONTH, 3);		
//		Date previa = DateUtils.add(desde, Calendar.MONTH, -3);
//		
//		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p, previa, desde);
//		repoTrans.makePersistent(compra1);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, desde, hasta);
//			repoTrans.makePersistent(compra);
//			PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p, desde, hasta);
//			repoTrans.makePersistent(pago);
//		}
//		assertTrue(MoneyHelper.equals(repo.getSaldoDeProveedorHasta(p, previa), BigDecimal.ZERO));
//		assertTrue(MoneyHelper.equals(repo.getSaldoDeProveedorHasta(p, desde), compra1.getMonto().negate()));
//	}
	
	/*public void testTransaccionesEntreFechas() {
		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
		repo.makePersistent(p);
		
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.DAY_OF_MONTH, 4);
		cal.set(Calendar.MONTH, 4);
		cal.set(Calendar.YEAR, 2008);
		Date desde = cal.getTime();
		Date hasta = DateUtils.add(desde, Calendar.MONTH, 3);		
		Date previa = DateUtils.add(desde, Calendar.MONTH, -3);
		Date futuro = DateUtils.add(hasta, Calendar.MONTH, 50);
		
		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p, previa, desde);
		repoTrans.makePersistent(compra1);
		
		for (int i=0;i<10;i++) {
			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, desde, hasta);
			repoTrans.makePersistent(compra);
			PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p, desde, hasta);
			repoTrans.makePersistent(pago);
		}
		assertEquals(1, repo.getTransaccionesDeProveedorEntreFechas(p, previa, desde).size());
		assertEquals(20, repo.getTransaccionesDeProveedorEntreFechas(p, desde, hasta).size());
		assertEquals(0, repo.getTransaccionesDeProveedorEntreFechas(p, hasta, futuro).size());
	}*/
	/*
	public void testUltimasTransacciones() {
		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
		repo.makePersistent(p);
		
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.DAY_OF_MONTH, 4);
		cal.set(Calendar.MONTH, 4);
		cal.set(Calendar.YEAR, 2008);
		Date desde = cal.getTime();
		Date hasta = DateUtils.add(desde, Calendar.MONTH, 3);		
		Date previa = DateUtils.add(desde, Calendar.MONTH, -3);
		
		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p, previa, desde);
		repoTrans.makePersistent(compra1);
		
		for (int i=0;i<10;i++) {
			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, desde, hasta);
			repoTrans.makePersistent(compra);
			PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p, desde, hasta);
			repoTrans.makePersistent(pago);
		}
		
		assertEquals(10, repo.getUltimasTransaccionesDeProveedor(p, 5).size());
	}*/
	
//	public void testPagosDeProveedorEntreFechas() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		Calendar cal = Calendar.getInstance();
//		cal.set(Calendar.DAY_OF_MONTH, 4);
//		cal.set(Calendar.MONTH, 4);
//		cal.set(Calendar.YEAR, 2008);
//		Date desde = cal.getTime();
//		Date hasta = DateUtils.add(desde, Calendar.MONTH, 3);		
//		Date previa = DateUtils.add(desde, Calendar.MONTH, -3);
//		
//		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p, previa, desde);
//		repoTrans.makePersistent(compra1);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, desde, hasta);
//			repoTrans.makePersistent(compra);
//			PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p, desde, hasta);
//			repoTrans.makePersistent(pago);
//		}
//		assertEquals(10, repo.getPagosDeProveedorEntreFechas(p, desde, hasta).size());
//		assertEquals(0, repo.getPagosDeProveedorEntreFechas(p, previa, desde).size());
//	}
//	
//	public void testComprasAProveedor() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		Calendar cal = Calendar.getInstance();
//		cal.set(Calendar.DAY_OF_MONTH, 4);
//		cal.set(Calendar.MONTH, 4);
//		cal.set(Calendar.YEAR, 2008);
//		Date desde = cal.getTime();
//		Date hasta = DateUtils.add(desde, Calendar.MONTH, 3);		
//		Date previa = DateUtils.add(desde, Calendar.MONTH, -3);
//		
//		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p, previa, desde);
//		repoTrans.makePersistent(compra1);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, desde, hasta);
//			repoTrans.makePersistent(compra);
//			PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p, desde, hasta);
//			repoTrans.makePersistent(pago);
//		}
//		assertEquals(11, repo.getComprasAProveedor(p).size());
//	}
//	
//	public void testComprasAProveedorEntreFechas() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		
//		Calendar cal = Calendar.getInstance();
//		cal.set(Calendar.DAY_OF_MONTH, 4);
//		cal.set(Calendar.MONTH, 4);
//		cal.set(Calendar.YEAR, 2008);
//		Date desde = cal.getTime();
//		Date hasta = DateUtils.add(desde, Calendar.MONTH, 3);		
//		Date previa = DateUtils.add(desde, Calendar.MONTH, -3);
//		
//		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p, previa, desde);
//		repoTrans.makePersistent(compra1);
//		
//		for (int i=0;i<10;i++) {
//			CompraAProveedorInmutable compra = RandomDataGenerator.getRandomCompraAProveedor(p, desde, hasta);
//			repoTrans.makePersistent(compra);
//			PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoEfectivoFromProveedor(p, desde, hasta);
//			repoTrans.makePersistent(pago);
//		}
//		assertEquals(10, repo.getComprasAProveedorEntreFechas(p, desde, hasta).size());
//		assertEquals(11, repo.getComprasAProveedorEntreFechas(p, previa, hasta).size());
//		assertEquals(1, repo.getComprasAProveedorEntreFechas(p, previa, desde).size());		
//	}
//	
//	public void testCuentaCorrienteUltimosMovimientosConProveedorSinMovimientos() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		HibernateUtil.closeSession();
//		CCProveedor cc = repo.getCuentaCorrienteConUltimosMovimientos(p);
//		assertTrue(MoneyHelper.equals(BigDecimal.ZERO, cc.getSaldoActual()));
//		CompraAProveedorInmutable compra1 = RandomDataGenerator.getRandomCompraAProveedor(p);
//		repoTrans.makePersistent(compra1);
//		HibernateUtil.closeSession();		
//		repoTrans.eliminarTransaccion(compra1);
//		HibernateUtil.closeSession();
//		cc = repo.getCuentaCorrienteConUltimosMovimientos(p);
//		assertTrue(MoneyHelper.equals(BigDecimal.ZERO, cc.getSaldoActual()));
//	}
//	
//	public void testCuentaCorrienteEnreFechas() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		HibernateUtil.closeSession();
//		
//		Calendar cal = Calendar.getInstance();
//		cal.set(Calendar.DAY_OF_MONTH, 1);
//		cal.set(Calendar.MONTH, 6);
//		cal.set(Calendar.YEAR, 2008);
//		
//		Date primeroJulio = cal.getTime();
//		
//		CompraAProveedorInmutable compra = new CompraAProveedorInmutable(cal.getTime(), p, "123", new BigDecimal("100"), new BigDecimal("100"), new BigDecimal("100"), new BigDecimal("100"), new BigDecimal("400"));
//		repoTrans.makePersistent(compra);
//		HibernateUtil.closeSession();
//		assertTrue(MoneyHelper.equals(new BigDecimal("-400"), repo.getSaldoDeProveedor(p)));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-400"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoActual()));
//		assertTrue(MoneyHelper.equals(BigDecimal.ZERO, repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoPrevio()));
//		
//		cal.set(Calendar.DAY_OF_MONTH, 2);
//		Date segundoJulio = cal.getTime();
//		
//		PagoAProveedorInmutable pago = new PagoAProveedorInmutable(p, cal.getTime(), "Notas", new BigDecimal("234"), "123");
//		repoTrans.makePersistent(pago);
//		HibernateUtil.closeSession();
//		assertTrue(MoneyHelper.equals(new BigDecimal("-166"), repo.getSaldoDeProveedor(p)));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-166"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoActual()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-400"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(BigDecimal.ZERO, repo.getCuentaCorrienteEntreFechas(p, primeroJulio, cal.getTime()).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-166"), repo.getCuentaCorrienteEntreFechas(p, primeroJulio, cal.getTime()).getSaldoActual()));
//		
//		cal.set(Calendar.DAY_OF_MONTH, 3);
//		Date terceroJulio = cal.getTime();
//		
//		CompraAProveedorInmutable compra2 = new CompraAProveedorInmutable(cal.getTime(), p, "123", new BigDecimal("120"), new BigDecimal("100"), new BigDecimal("110"), new BigDecimal("100"), new BigDecimal("430"));
//		repoTrans.makePersistent(compra2);
//		HibernateUtil.closeSession();
//		assertTrue(MoneyHelper.equals(new BigDecimal("-596"), repo.getSaldoDeProveedor(p)));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-596"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoActual()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-166"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(BigDecimal.ZERO, repo.getCuentaCorrienteEntreFechas(p, primeroJulio, cal.getTime()).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-400"), repo.getCuentaCorrienteEntreFechas(p, segundoJulio, terceroJulio).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-596"), repo.getCuentaCorrienteEntreFechas(p, segundoJulio, terceroJulio).getSaldoActual()));
//		
//		
//		PagoAProveedorInmutable pago2 = new PagoAProveedorInmutable(p, cal.getTime(), "Notas", new BigDecimal("100"), "123");
//		repoTrans.makePersistent(pago2);
//		HibernateUtil.closeSession();
//		assertTrue(MoneyHelper.equals(new BigDecimal("-496"), repo.getSaldoDeProveedor(p)));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-496"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoActual()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-166"), repo.getCuentaCorrienteEntreFechas(p, cal.getTime(), cal.getTime()).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(BigDecimal.ZERO, repo.getCuentaCorrienteEntreFechas(p, primeroJulio, cal.getTime()).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-400"), repo.getCuentaCorrienteEntreFechas(p, segundoJulio, terceroJulio).getSaldoPrevio()));
//		assertTrue(MoneyHelper.equals(new BigDecimal("-496"), repo.getCuentaCorrienteEntreFechas(p, segundoJulio, terceroJulio).getSaldoActual()));
//	}
//	
//	public void testEliminarPagoConCheque() {
//		Proveedor p = new Proveedor("Maria", "Notas", TipoDeProveedor.MERCADERIA, "320548");
//		repo.makePersistent(p);
//		HibernateUtil.closeSession();
//		
//		PagoAProveedorInmutable pago = RandomDataGenerator.getRandomPagoConChequeToProveedor(p);
//		repoTrans.makePersistentAndUpdateCheque(pago);
//		HibernateUtil.closeSession();
//		
//		HibChequesRepository repoCheques = new HibChequesRepository();
//		
//		assertEquals(1, repo.getTransaccionesDeProveedor(p).size());
//		assertEquals(0, repoCheques.getChequesPorEstado(EstadoDeCheque.EN_POSESION_DE_FENIX).size());
//		assertEquals(0, repoCheques.getChequesPorEstado(EstadoDeCheque.DEPOSITADO).size());
//		assertEquals(1, repoCheques.getChequesPorEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR).size());
//		HibernateUtil.closeSession();
//		
//		repoTrans.eliminarTransaccion(pago);
//		HibernateUtil.closeSession();
//		assertEquals(0, repo.getTransaccionesDeProveedor(p).size());
//		assertEquals(1, repoCheques.getChequesPorEstado(EstadoDeCheque.EN_POSESION_DE_FENIX).size());
//		assertEquals(0, repoCheques.getChequesPorEstado(EstadoDeCheque.DEPOSITADO).size());
//		assertEquals(0, repoCheques.getChequesPorEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR).size());
//		
//	}
	
	public void testFakeTest() {
		
	}
	
}
