package org.ziggurat.fenix.web;

import java.util.Calendar;

import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.Panel;
import org.ziggurat.fenix.web.cheques.ListadoDeCheques;
import org.ziggurat.fenix.web.clientes.ListadoDeClientes;
import org.ziggurat.fenix.web.inicio.Inicio;
import org.ziggurat.fenix.web.matafuegos.ListadoDeMatafuegos;
import org.ziggurat.fenix.web.proveedores.ListadoDeProveedores;
import org.ziggurat.fenix.web.resumenes.ResumenMensualPage;

abstract public class AbstractFenixPage extends WebPage {

	protected LinkToProveedores linkToProveedores = new LinkToProveedores("proveedores");
	protected LinkToResumenes linkToResumenes = new LinkToResumenes("resumenes");
	protected LinkToCheques linkToCheques = new LinkToCheques("cheques");
	protected LinkToClientes linkToClientes = new LinkToClientes("clientes");
	protected LinkToMatafuegos linkToMatafuegos = new LinkToMatafuegos("matafuegos");
    protected LinkToInicio linkToInicio = new LinkToInicio("inicio");

    
    public final class LinkToInicio extends Link<Object> {
        public LinkToInicio(String id) {
            super(id);
        }

        @Override
		public void onClick() {
			setResponsePage(Inicio.class);
		}
    }


	public final class LinkToMatafuegos extends Link<Object> {
		
		public LinkToMatafuegos(String id) {
			super(id);
		}
		
		@Override
		public void onClick() {
			setResponsePage(ListadoDeMatafuegos.class);
		}
	}
	
	//@AuthorizeAction(action = Action.RENDER, roles = "FENIX")
	public final class LinkToProveedores extends Link<ListadoDeProveedores>  {

		public LinkToProveedores(String id) {
			super(id);
		}
		
		@Override
		public void onClick() {
			setResponsePage(ListadoDeProveedores.class);
		}
		
	}
	
	//@AuthorizeAction(action = "ENABLE", roles={FenixRoles.CLIENTES})
	public final class LinkToClientes extends Link<ListadoDeClientes> {
		
		public LinkToClientes(String id) {
			super(id);
		}
		
		@Override
		public void onClick() {
			setResponsePage(ListadoDeClientes.class);
		}
	}
	
	public final class LinkToResumenes extends Link<ResumenMensualPage> {
		
		public LinkToResumenes(String id) {
			super(id);
		}
		
		@Override
		public void onClick() {
			Calendar cal = Calendar.getInstance();
			setResponsePage(new ResumenMensualPage(cal.get(Calendar.MONTH) + 1, cal.get(Calendar.YEAR)));
		}
	}
	
	public final class LinkToCheques extends Link<ListadoDeCheques> {
		
		public LinkToCheques(String id) {
			super(id);
		}
		
		@Override
		public void onClick() {
			setResponsePage(ListadoDeCheques.class);
		}
	}
	
	public AbstractFenixPage() {
		
		add(new LoggedAsPanel("loggedAs"));
		add(linkToProveedores);
		add(linkToResumenes);
		add(linkToCheques);
		add(linkToClientes.setVisible(false));
		add(linkToMatafuegos.setVisible(false));
        add(linkToInicio.setVisible(false));
		add(getSubMenuPanel());
	}
	
	abstract public Panel getSubMenuPanel();
	
}
