package org.ziggurat.fenix.web.cheques;

import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.database.hibernate.HibChequesRepository;
import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.web.FenixLinkWithConfirmation;
import org.ziggurat.fenix.web.FenixPagingNavigator;

public class ListadoDeCheques extends AbstractChequesPage {

	private static Logger log = Logger.getLogger("fenix");
	private EstadoDeCheque estado = EstadoDeCheque.EN_POSESION_DE_FENIX;

	public EstadoDeCheque getEstado() {
		return estado;
	}

	public ListadoDeCheques() {
		super();
		init(EstadoDeCheque.EN_POSESION_DE_FENIX);
		log.info("Entrando a pagina de listado de cheques en posesion");		
	}
	
	public ListadoDeCheques(EstadoDeCheque estado) {
		this.estado = estado;
		init(estado);
	}
	
	public void init(EstadoDeCheque estado) {
		String header = "";
		IDataProvider<Cheque> dataProvider = new ChequesDataProvider(estado);
		if (EstadoDeCheque.EN_POSESION_DE_FENIX.equals(estado)) {
			header = "en posesion de fenix";
		} else if (EstadoDeCheque.ENTREGADO_A_PROVEEDOR.equals(estado)) {
			header = "entregados a proveedor";
		} else if (EstadoDeCheque.DEPOSITADO.equals(estado)) {
			header = "depositados o cobrados";
		}
		
		add(new Label("pageHeader", header));
		final HibChequesRepository repoCheques = new HibChequesRepository();
		//List<Cheque> listaCheques = repoCheques.getChequesPorEstado(estado);
		final DataView<Cheque> dataView = new DataView<Cheque>("view", dataProvider, 10) {
			private static final long serialVersionUID = 1L;

			public void populateItem(final Item<Cheque> item) {
				final Cheque cheque = item.getModelObject();
				
				final ModalWindow modal;
				item.add(modal = new ModalWindow("infoCheque"));
				modal.setCssClassName(ModalWindow.CSS_CLASS_GRAY);
				modal.setContent(new DetalleCheque(modal.getContentId(), cheque));
				modal.setTitle("Detalles del cheque");
				modal.setCookieName("cheque" + cheque.hashCode());
				modal.setWidthUnit("em");
				modal.setHeightUnit("em");
				modal.setInitialWidth(35);
				modal.setInitialHeight(23);
				modal.setResizable(false);
				
				item.add(new AjaxLink("numeroLink") {
					@Override
					public void onClick(AjaxRequestTarget target) {
						modal.show(target);
					}
				}.add(new Label("numero", cheque.getNumeroCheque())));
				item.add(new Label("cuit", cheque.getCUIT()));
				item.add(new Label("banco", cheque.getBanco()));
				item.add(new Label("notas", cheque.getDescripcion()));
				item.add(new Label("monto", FenixFormat.getMoneda(cheque.getMonto())));
				Link depositar = new Link("depositarLink") {
					@Override
					public void onClick() {
						cheque.setEstado(EstadoDeCheque.DEPOSITADO);
						repoCheques.makePersistent(cheque);
						setResponsePage(ListadoDeCheques.class);
					}
					
					@Override
					public boolean isVisible() {
						return getEstado().equals(EstadoDeCheque.EN_POSESION_DE_FENIX);
					}
				};
				Link otroDestino = new Link("otroDestino") {
					@Override
					public void onClick() {						
						setResponsePage(new AccionSobreChequePage(cheque));
					}
					
					@Override
					public boolean isVisible() {
						return getEstado().equals(EstadoDeCheque.EN_POSESION_DE_FENIX);
					}
				};
				item.add(new FenixLinkWithConfirmation("eliminarCheque", "Desea eliminar este cheque?") {
					@Override
					public void onClick() {
						repoCheques.eliminarCheque(cheque);
						setResponsePage(ListadoDeCheques.class);
					}
				});
				item.add(depositar);
				item.add(otroDestino);
				item.add(new AttributeModifier("class", true,
						new AbstractReadOnlyModel() {
							public Object getObject() {
								return (item.getIndex() % 2 == 1) ? "even" : "odd";
							}
						}));
			}
		};
		add(dataView);
		add(new FenixPagingNavigator("navigator", dataView));
	}
		
}
