package org.ziggurat.fenix.web.cheques;

import org.apache.log4j.Logger;
import org.apache.wicket.extensions.markup.html.form.DateTextField;
import org.apache.wicket.extensions.yui.calendar.DatePicker;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.util.convert.IConverter;
import org.ziggurat.fenix.common.database.hibernate.HibChequesRepository;
import org.ziggurat.fenix.common.database.hibernate.HibernateUtil;
import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.web.FenixBigDecimalConverter;
import org.ziggurat.fenix.web.FocusOnBehavior;

public class NuevoCheque extends AbstractChequesPage {

	private static Logger log = Logger.getLogger("fenix");
	private Cheque cheque = new Cheque();

	public NuevoCheque() {
		log.info("Entrando a pagina de nuevo cheque");
		cheque.setEstado(EstadoDeCheque.EN_POSESION_DE_FENIX);
		add(new NuevoChequeForm("nuevoChequeForm"));
	}

	private class NuevoChequeForm extends Form {

		public NuevoChequeForm(String id) {
			super(id);

			CompoundPropertyModel model = new CompoundPropertyModel(cheque);
			this.setModel(model);

			TextField<Cheque> numeroCheque = new TextField<Cheque>("numeroCheque");
			numeroCheque.setRequired(true);
			numeroCheque.add(new FocusOnBehavior());
			add(numeroCheque);

			TextField<Cheque> entregadoPor = new TextField<Cheque>("entregadoPor");
			add(entregadoPor);

			TextField<Cheque> CUIT = new TextField<Cheque>("CUIT");
			add(CUIT);

			TextField<Cheque> banco = new TextField<Cheque>("banco");
			banco.setRequired(true);
			add(banco);

			TextField<Cheque> aLaOrdenDe = new TextField<Cheque>("aLaOrdenDe");
			add(aLaOrdenDe);

			TextField<Cheque> monto = new TextField<Cheque>("monto") {
				
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}
			};
			monto.setRequired(true);
			add(monto);

			TextField<Cheque> descripcion = new TextField<Cheque>("descripcion");
			add(descripcion);

			DateTextField fecha = new DateTextField("fecha", FenixFormat.DATE_FORMAT.toPattern());
			fecha.setRequired(true);
			fecha.add(new DatePicker());
			add(fecha);

			DateTextField fechaDeCobro = new DateTextField("fechaDeCobro", FenixFormat.DATE_FORMAT.toPattern());
			fechaDeCobro.add(new DatePicker());
			add(fechaDeCobro);
			
			/*DropDownChoice estado=new DropDownChoice("estado", Arrays.asList(EstadoDeCheque.values()));
			add(estado);*/

			add(new FeedbackPanel("feedback"));

		}

		@Override
		protected void onSubmit() {
			HibernateUtil.beginTransaction();
			HibChequesRepository repo = new HibChequesRepository();
			repo.makePersistent(cheque);
			setResponsePage(ListadoDeCheques.class);
			HibernateUtil.commitTransaction();
		}

	}

}
