package org.ziggurat.fenix.web.clientes;

import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.web.FenixPagingNavigator;
import org.ziggurat.fenix.web.matafuegos.ListadoDeMatafuegos;
import org.ziggurat.fenix.web.matafuegos.NuevoMatafuego;

public class ListadoDeClientes extends AbstractClientesPage {

	private static Logger log = Logger.getLogger("fenix");
	private IDataProvider<Cliente> dataProvider;
	
	public ListadoDeClientes() {
		dataProvider = new ClientesDataProvider();
		buildPage("Listado de clientes");
	}
	
	public ListadoDeClientes(String busqueda) {
		dataProvider = new BusquedaClientesDataProvider(busqueda);
		buildPage("Lista de clientes que coinciden con '" + busqueda + "'");
	}
	
	public void buildPage(String pageHeader) {
		add(new BuscarClientesPanel("buscarClientePanel"));
		add(new Label("pageTitle", pageHeader));
		
		final DataView<Cliente> dataView = new DataView<Cliente>("view", dataProvider, 10) {

			@Override
			protected void populateItem(final Item<Cliente> item) {
				final Cliente cliente = item.getModelObject();
				
				item.add(new Label("nombre", cliente.getNombre()));
				item.add(new Label("domicilio", cliente.getDomicilio()));
				item.add(new Label("telefono", cliente.getTelefono()));
				item.add(new Link<Cliente>("verMatafuegos") {

					@Override
					public void onClick() {
						setResponsePage(new ListadoDeMatafuegos(cliente));
					}
					
				});
				item.add(new Link<Cliente>("nuevoMatafuego") {

					@Override
					public void onClick() {
						setResponsePage(new NuevoMatafuego(cliente));
					}
					
				});
				
				item.add(new AttributeModifier("class", true,
						new AbstractReadOnlyModel() {
							public Object getObject() {
								return (item.getIndex() % 2 == 1) ? "even" : "odd";
							}
						}));
			}
			
		};
	
		add(dataView);
		add(new FenixPagingNavigator("navigator", dataView));
	}
	
}
