package org.ziggurat.fenix.web.matafuegos;

import org.apache.commons.lang.StringUtils;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.modelo.matafuego.Matafuego;
import org.ziggurat.fenix.common.services.matafuegos.MatafuegosService;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.web.FenixPagingNavigator;

public class ListadoDeMatafuegos extends AbstractMatafuegosPage {

	private IDataProvider<Matafuego> dataProvider;
	private Cliente cliente;
	
	public ListadoDeMatafuegos() {
		add(new Label("header", "Listado de matafuegos"));
		dataProvider = new MatafuegosDataProvider();
		buildPage();
	}
	
	public ListadoDeMatafuegos(Cliente cliente) {
		this.cliente = cliente;
		add(new Label("header", "Listado de matafuegos de " + cliente.getNombre()));
		dataProvider = new MatafuegosDeClienteDataProvider(cliente); 
		buildPage();
	}
	
	private void buildPage() {
		
		final DataView<Matafuego> dataView = new DataView<Matafuego>("view", dataProvider, 10) {

			@Override
			protected void populateItem(Item<Matafuego> item) {
				final Matafuego matafuego = item.getModelObject();
                MatafuegosService service = new MatafuegosService();
				item.add(new Label("nroAparato", matafuego.getNroAparato()));
				String cliente;
				Link linkToCliente = new Link("linkCliente") {
					@Override
					public void onClick() {
						setResponsePage(new ListadoDeMatafuegos(matafuego.getCliente()));
					}
				};
				if (matafuego.getCliente() != null) {
					cliente = matafuego.getCliente().getNombre();
					if (getCliente() != null) {
						linkToCliente.setVisible(false);
					}
				} else {
					cliente = matafuego.getNombre();
					linkToCliente.setVisible(false);
				}
				item.add(linkToCliente);
				item.add(new Label("nombreCliente", cliente));
				item.add(new Label("idInterna", StringUtils.defaultIfEmpty(matafuego.getIdInterna(), "")));
				item.add(new Label("ubicacion", StringUtils.defaultIfEmpty(matafuego.getUbicacion(), "")));
				item.add(new Label("capacidad", String.valueOf(matafuego.getCapacidad())));            
				//PruebaHidraulica ph = matafue.getUltimaPruebaHidraulica(matafuego);
				if (matafuego.getProximaPruebaHidraulica() != null) {
                    item.add(new Label("ultimaPH", FenixFormat.getFecha(matafuego.getProximaPruebaHidraulica())));
				} else {
					item.add(new Label("ultimaPH", "No se realizaron"));
				}
				//MantenimientoMatafuego mant = repoMatafuegos.getUltimoMantenimiento(matafuego);
				if (matafuego.getProximoMantenimiento() != null) {
					item.add(new Label("ultimoMantenimiento", FenixFormat.getFecha(matafuego.getProximoMantenimiento())));
				} else {
					item.add(new Label("ultimoMantenimiento", "No se realizaron"));
				}
			}
			
		};
		
		add(dataView);
		add(new FenixPagingNavigator("navigator", dataView));
	}

	public Cliente getCliente() {
		return cliente;
	}
	
}
