package org.ziggurat.fenix.web.matafuegos;

import java.util.Arrays;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.ajax.markup.html.IndicatingAjaxLink;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.Model;
import org.hibernate.exception.ConstraintViolationException;
import org.ziggurat.fenix.common.modelo.clientes.Cliente;
import org.ziggurat.fenix.common.modelo.matafuego.Matafuego;
import org.ziggurat.fenix.common.modelo.matafuego.TipoMatafuego;
import org.ziggurat.fenix.common.services.matafuegos.MatafuegosService;
import org.ziggurat.fenix.web.FocusOnBehavior;
import org.ziggurat.fenix.web.clientes.ModalWindowSeleccionarCliente;

public class NuevoMatafuego extends AbstractMatafuegosPage {

	private Matafuego matafuego;
	final ModalWindowSeleccionarCliente mw = new ModalWindowSeleccionarCliente("mw");
	
	public NuevoMatafuego() {		
		matafuego = new Matafuego();
		final NuevoMatafuegoForm form = new NuevoMatafuegoForm("nuevoMatafuegoForm");
		form.setOutputMarkupId(true);
		add(form);
		add(mw);
		
	}
	
	public NuevoMatafuego(Cliente cliente) {
		matafuego = new Matafuego();
		matafuego.setCliente(cliente);
		matafuego.setNombre(cliente.getNombre());
	}
	
	private class NuevoMatafuegoForm extends Form {
				
		TextField<Matafuego> nroAparato = new TextField<Matafuego>("nroAparato");
		TextField<Matafuego> marca = new TextField<Matafuego>("marca");
		TextField<Matafuego> idInterna = new TextField<Matafuego>("idInterna");
		TextField<Matafuego> ubicacion = new TextField<Matafuego>("ubicacion");
		TextField<Matafuego> capacidad = new TextField<Matafuego>("capacidad");
		TextField<Matafuego> anoFabricacion = new TextField<Matafuego>("anoFabricacion");
		TextField<Matafuego> nombre = new TextField<Matafuego>("nombre");
		TextField<Matafuego> direccion = new TextField<Matafuego>("direccion");
		DropDownChoice<TipoMatafuego> tipoMatafuego = new DropDownChoice<TipoMatafuego>("tipoMatafuego", Arrays.asList(TipoMatafuego.values()));
		Label selectedCliente = new Label("cliente", new Model(""));
		IndicatingAjaxLink linkSelectCliente = new IndicatingAjaxLink("linkSelectCliente") {

			@Override
			public void onClick(AjaxRequestTarget target) {
				mw.setCliente(null);
				mw.show(target);
			}
		};
		
		public NuevoMatafuegoForm(String id) {
			super(id);
			
			CompoundPropertyModel<Matafuego> model = new CompoundPropertyModel<Matafuego>(matafuego);
			setModel(model);
			
			
			final Label labelLinkSeleccionarCliente = new Label("labelLinkSelectCliente", "Seleccionar cliente");
			labelLinkSeleccionarCliente.setOutputMarkupPlaceholderTag(true);
			
			mw.setWindowClosedCallback(new ModalWindow.WindowClosedCallback() {

				@Override
				public void onClose(AjaxRequestTarget target) {
					if (mw.getCliente() != null) {
						matafuego.setCliente(mw.getCliente());
						selectedCliente.setDefaultModelObject(mw.getCliente().toString());
						labelLinkSeleccionarCliente.setDefaultModelObject("Cambiar");						
					} else {
						selectedCliente.setDefaultModelObject("No seleccionado");						
					}
					target.addComponent(selectedCliente);
					target.addComponent(labelLinkSeleccionarCliente);
				}
				
			});
			
			selectedCliente.setOutputMarkupPlaceholderTag(true);
			 
			add(linkSelectCliente.add(labelLinkSeleccionarCliente));
			add(selectedCliente);
			
			nroAparato.setRequired(true);
			nroAparato.add(new FocusOnBehavior());
			add(nroAparato);
			
			add(marca);
			add(idInterna);
			add(ubicacion);
			
			capacidad.setRequired(true);
			add(capacidad);
						
			anoFabricacion.setRequired(true);
			add(anoFabricacion);
						
			//nombre.setRequired(true);
			add(nombre);
						
			add(direccion);
						
			tipoMatafuego.setRequired(true);
			add(tipoMatafuego);					
			
			add(new FeedbackPanel("feedback"));
		}
		
		@Override
		protected void onSubmit() {
			MatafuegosService service = new MatafuegosService();
			try {
				if (matafuego.getCliente() == null) {
					error("Debe seleccionar un cliente");
					return;
				}
				service.save(matafuego);
				setResponsePage(ListadoDeMatafuegos.class);
			} catch (ConstraintViolationException e) {
				error("Ya exite un matafuego registrado con ese numero");
			}
		}
		
		
	}
	
}
