package org.ziggurat.fenix.web.proveedores;

import java.util.Date;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.database.hibernate.HibProveedorRepository;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.modelo.proveedores.CCProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.CompraAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.LineaCCProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.PagoAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.common.modelo.transacciones.TipoDeTransaccion;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.web.FenixLinkWithConfirmation;
import org.ziggurat.fenix.web.FenixPanelSelectorDeFechas;

public class CCProveedorPage extends AbstractProveedoresPage {
	
	private static Logger log = Logger.getLogger("fenix"); 
	private Proveedor proveedor;
	private Date fechaInicial = null;
	private Date fechaFinal = null;
	
	public CCProveedorPage(final Proveedor proveedor) {
		HibProveedorRepository repo = new HibProveedorRepository();
		this.proveedor = proveedor;
		buildPage(repo.getCuentaCorrienteConUltimosMovimientos(proveedor));
	}
	
	public CCProveedorPage(final Proveedor proveedor, Date desde, Date hasta) {
		HibProveedorRepository repo = new HibProveedorRepository();
		this.proveedor = proveedor;
		this.fechaInicial = desde;
		this.fechaFinal = hasta;
		buildPage(repo.getCuentaCorrienteEntreFechas(proveedor, desde, hasta));
	}
	
	private void buildPage(CCProveedor cc) {
		log.info("Entrando a pagina de CC de " + proveedor.toString());
		
		add(new FenixPanelSelectorDeFechas("panelFechas", "Ver movimientos entre dos fechas:") {
				public void doAction(Date desde, Date hasta) {
					setResponsePage(new CCProveedorPage(proveedor, desde, hasta));					
				}
			}
		);
		add(new Link("modificarDatos") {
			@Override
			public void onClick() {
				setResponsePage(new NuevoProveedor(proveedor));				
			}
		});
		add(new Label("nombreProveedor", proveedor.getNombre()));
		String captionFechaInicial = "SALDO HASTA EL ";
		String labelFechaInicial = "";
		String captionFechaFinal = "";
		String labelFechaFinal = "";
		String titulo = "Ultimos movimientos";
		
		if (fechaInicial != null && fechaFinal != null) {
			titulo = String.format("Movimientos entre %s y %s", FenixFormat.getFecha(this.fechaInicial), FenixFormat.getFecha(this.fechaFinal));
			labelFechaInicial = FenixFormat.getFecha(this.fechaInicial);
			captionFechaFinal = "SALDO AL DIA ";
			labelFechaFinal = FenixFormat.getFecha(this.fechaFinal);
		} else {
			if (cc.getLineasDeCC().size() > 0) {
				labelFechaInicial = FenixFormat.getFecha(cc.getLineasDeCC().get(0).getFecha());				
			}
			labelFechaFinal = "";
			captionFechaFinal = "SALDO";
		}
		add(new Label("titulo", titulo));
		Link verUltimos = new Link("verUltimos") {
			@Override
			public void onClick() {
				setResponsePage(new CCProveedorPage(proveedor));				
			}
		}; 
		add(verUltimos);
		if ("Ultimos movimientos".equals(titulo)) {
			verUltimos.setVisible(false);
		}
		add(new Label("fechaSaldoInicial", captionFechaInicial + labelFechaInicial));
		add(new Label("fechaSaldoFinal", captionFechaFinal + labelFechaFinal));
		add(new Label("saldoInicial",FenixFormat.getMoneda(cc.getSaldoPrevio())));
		add(new Label("saldoActual", FenixFormat.getMoneda(cc.getSaldoActual())));
		
		
		final DataView dataView = new DataView("transacciones", new ListDataProvider(cc.getLineasDeCC())) {
			private static final long serialVersionUID = 1L;

			protected void populateItem(final Item item) {
				final LineaCCProveedor linea = (LineaCCProveedor) item.getModelObject();
				item.add(new Label("fecha", FenixFormat.getFecha(linea.getFecha())));				
				item.add(new Label("debe", (String) ObjectUtils.defaultIfNull(FenixFormat.getMoneda(linea.getDebe()), "")));
				item.add(new Label("haber", (String) ObjectUtils.defaultIfNull(FenixFormat.getMoneda(linea.getHaber()), "")));
				item.add(new Label("saldo", FenixFormat.getMoneda(linea.getSaldo())));
				item.add(new Label("comentario", StringUtils.abbreviate(linea.getComentario(), 10)));
				if (linea.getTransaccion().getTipoDeTransaccion().equals(TipoDeTransaccion.COMPRA_A_PROVEEDOR)) {
					final CompraAProveedorInmutable compra = (CompraAProveedorInmutable) linea.getTransaccion();
					item.add(new LineaDeCCCompraPanel("detalleMovimiento", compra));
					//item.add(new DetalleDeTransaccionAjaxLink("comprobanteLink", compra).add(new Label("comprobante", String.format("Compra a proveedor (Factura %s)", compra.getNumero()))));
				} else if (linea.getTransaccion().getTipoDeTransaccion().equals(TipoDeTransaccion.PAGO_A_PROVEEDOR_CHEQUE)) {
					final PagoAProveedorInmutable pago = (PagoAProveedorInmutable) linea.getTransaccion();
					item.add(new LineaDeCCPagoChequePanel("detalleMovimiento", pago));
					//item.add(new DetalleDeTransaccionAjaxLink("comprobanteLink", pago).add(new Label("comprobante", String.format("Pago con cheque ( Cheque: %s )", pago.getCheque().getNumeroCheque()))));
				} else if (linea.getTransaccion().getTipoDeTransaccion().equals(TipoDeTransaccion.PAGO_A_PROVEEDOR_CONTADO)) {
					final PagoAProveedorInmutable pago = (PagoAProveedorInmutable) linea.getTransaccion();
					item.add(new LineaDeCCPagoEfectivoPanel("detalleMovimiento"));
					//item.add(new DetalleDeTransaccionAjaxLink("comprobanteLink", pago).add(new Label("comprobante", String.format("Pago efectivo (Recibo: %s)", StringUtils.defaultIfEmpty(pago.getNroRecibo(),"SIN NUMERO")))));
				}				
				item.add(new AttributeModifier("class", true, new AbstractReadOnlyModel() {
					public Object getObject() {
						return (item.getIndex() % 2 == 1) ? "even" : "odd";
					}
				}));
				item.add(new FenixLinkWithConfirmation("eliminarTransaccion", "Desea eliminar este movimiento?") {
					@Override
					public void onClick() {
						HibTransaccionConProveedorRepository repo = new HibTransaccionConProveedorRepository();
						repo.eliminarTransaccion(linea.getTransaccion());
						setResponsePage(new CCProveedorPage(proveedor));
					}
				});
			}
		};
		
		add(new Link("nuevoPagoEfectivoLink") {
			@Override
			public void onClick() {
				setResponsePage(new NuevoPagoEfectivo(proveedor));
			}
		});
		add(new Link("nuevoPagoChequeLink") {
			@Override
			public void onClick() {
				setResponsePage(new NuevoPagoConCheque(proveedor));
			}
		});
		add(new Link("nuevaFacturaLink") {
			@Override
			public void onClick() {
				setResponsePage(new NuevaFactura(proveedor));
			}
		});
		
		add(dataView);		
	}

}

