package org.ziggurat.fenix.web.proveedores;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxButton;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.ziggurat.fenix.common.modelo.proveedores.CategoriaDeProveedor;
import org.ziggurat.fenix.common.services.proveedores.CategoriasDeProveedorService;
import org.ziggurat.fenix.web.FocusOnBehavior;

public class ListadoDeCategorias extends AbstractProveedoresPage {
	
	private WebMarkupContainer datacontainer = new WebMarkupContainer("data");
	
	public ListadoDeCategorias() {
		
		CategoriasDeProveedorService catService = new CategoriasDeProveedorService();
		IDataProvider<CategoriaDeProveedor> dataProvider = new ListDataProvider<CategoriaDeProveedor>(catService.getCategorias());
		
		
        datacontainer.setOutputMarkupId(true);
        add(datacontainer);
		
		final DataView<CategoriaDeProveedor> dataView = new DataView<CategoriaDeProveedor>("view", dataProvider) {
			
			public void populateItem(final Item<CategoriaDeProveedor> item) {
				final CategoriaDeProveedor categoria = item.getModelObject();
				
				item.add(new Label("itemNombre", categoria.getNombre()));
				item.add(new Label("itemDescripcion", categoria.getDescripcion()));				
				item.add(new AttributeModifier("class", true,
						new AbstractReadOnlyModel() {
							public Object getObject() {
								return (item.getIndex() % 2 == 1) ? "even" : "odd";
							}
						}));
			}
		};
		datacontainer.add(dataView);		
		add(new NuevaCategoriaForm("form"));
	}
	
	private class NuevaCategoriaForm extends Form<CategoriaDeProveedor> {

		CategoriasDeProveedorService catService = new CategoriasDeProveedorService();
		private CategoriaDeProveedor nuevaCategoria = new CategoriaDeProveedor();
		
		public NuevaCategoriaForm(String id) {
			super(id);
			
			CompoundPropertyModel<CategoriaDeProveedor> model = new CompoundPropertyModel<CategoriaDeProveedor>(nuevaCategoria);
			this.setModel(model);

			TextField<CategoriaDeProveedor> nombre = new TextField<CategoriaDeProveedor>("nombre");
			nombre.setRequired(true);
			nombre.add(new FocusOnBehavior());
			add(nombre);

			TextField<CategoriaDeProveedor> descripcion = new TextField<CategoriaDeProveedor>("descripcion");
			add(descripcion);

			final FeedbackPanel feedback = new FeedbackPanel("feedback");
	        feedback.setOutputMarkupId(true);
	        add(feedback);			
		}
		
		@Override
		protected void onSubmit() {
			catService.saveCategoria(nuevaCategoria);
			setResponsePage(ListadoDeCategorias.class);
		}

	}
	
}
