package org.ziggurat.fenix.web.proveedores;

import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.web.FenixPagingNavigator;

public class ListadoDeProveedores extends AbstractProveedoresPage {

	private static Logger log = Logger.getLogger("fenix");
	private IDataProvider<Proveedor> dataProvider;

	public ListadoDeProveedores() {
		add(new Label("pageTitle", "Listado de proveedores"));
		dataProvider = new ProveedoresDataProvider();
		initializePage();
	}
	
	public ListadoDeProveedores(String busqueda) {
		add(new Label("pageTitle", "Lista de proveedores que coinciden con '" + busqueda + "'"));
		dataProvider = new BusquedaProveedoresDataProvider(busqueda);
		initializePage();
	}
	
	private void initializePage() {
		log.info("Entrando a pagina ListadoDeProveedores");
		final DataView<Proveedor> dataView = new DataView<Proveedor>("view", dataProvider, 12) {
			private static final long serialVersionUID = 1L;

			public void populateItem(final Item<Proveedor> item) {
				final Proveedor proveedor = item.getModelObject();
				item.add(new Link("nombreLink") {
					@Override
					public void onClick() {
						setResponsePage(new CCProveedorPage(proveedor));
					}
				}.add(new Label("nombre", proveedor.getNombre())));
				
				item.add(new Label("tipo", proveedor.getCategoria().getNombre()));
				item.add(new Label("cuit", proveedor.getCUIT()));
				item.add(new Link("nuevoPagoEfectivoLink") {
					@Override
					public void onClick() {
						setResponsePage(new NuevoPagoEfectivo(proveedor));
					}
				});
				item.add(new Link("nuevoPagoChequeLink") {
					@Override
					public void onClick() {
						setResponsePage(new NuevoPagoConCheque(proveedor));
					}
				});
				item.add(new Link("nuevaFacturaLink") {
					@Override
					public void onClick() {
						setResponsePage(new NuevaFactura(proveedor));
					}
				});
				item.add(new AttributeModifier("class", true,
						new AbstractReadOnlyModel() {
							public Object getObject() {
								return (item.getIndex() % 2 == 1) ? "even" : "odd";
							}
						}));
			}
		};
		add(new BuscarProveedorPanel("buscarProveedorPanel"));
		add(dataView);
		add(new FenixPagingNavigator("navigator", dataView));
	}
}
