package org.ziggurat.fenix.web.proveedores;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;
import org.apache.wicket.extensions.markup.html.form.DateTextField;
import org.apache.wicket.extensions.yui.calendar.DatePicker;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.util.convert.IConverter;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.modelo.proveedores.CompraAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.common.util.MoneyHelper;
import org.ziggurat.fenix.web.FenixBigDecimalConverter;
import org.ziggurat.fenix.web.FocusOnBehavior;

public class NuevaFactura extends AbstractProveedoresPage {

	private Proveedor proveedor;
	private static Logger log = Logger.getLogger("fenix");

	public NuevaFactura(Proveedor proveedor) {
		log.info("Entrando a pagina de Nueva Factura " + proveedor.getNombre());
		this.proveedor = proveedor;
		add(new Label("nombreProveedor", proveedor.getNombre()));
		add(new NuevaFacturaForm("nuevaFacturaForm"));
	}

	private class NuevaFacturaForm extends Form {

		private Date fecha = Calendar.getInstance().getTime();
		private String numero;
		private BigDecimal netoSinIVA;
		private BigDecimal montoIVA;
		private BigDecimal noGravadoExento;
		private BigDecimal percepciones;
		private BigDecimal total;

		public NuevaFacturaForm(String id) {
			super(id);

			CompoundPropertyModel model = new CompoundPropertyModel(this);
			this.setModel(model);

			DateTextField dateTextField = new DateTextField("fecha", FenixFormat.DATE_FORMAT.toPattern());
			dateTextField.setRequired(true);
			DatePicker datePicker = new DatePicker();					
			dateTextField.add(new DatePicker());
			add(dateTextField);

			TextField numeroTF = new TextField("numero");
			numeroTF.setRequired(true);
			numeroTF.add(new FocusOnBehavior());
			add(numeroTF);

			final TextField netoSinIVATF = new TextField("netoSinIVA") {
				@Override
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}

			};
			netoSinIVATF.setRequired(true);
			add(netoSinIVATF);
			final TextField montoIVATF = new TextField("montoIVA") {
				@Override
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}

			};
			montoIVATF.setRequired(true);
			add(montoIVATF);
			final TextField noGravadoExentoTF = new TextField("noGravadoExento") {
				@Override
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}

			};
			noGravadoExentoTF.setRequired(true);
			add(noGravadoExentoTF);
			final TextField percepcionesTF = new TextField("percepciones") {
				@Override
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}

												
			};
			percepcionesTF.setRequired(true);
			add(percepcionesTF);
			TextField totalTF = new TextField("total") {
				@Override
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}
			};
			totalTF.setRequired(true);
			add(totalTF);

			add(new FeedbackPanel("feedback"));
		}

		@Override
		protected void onSubmit() {
			BigDecimal totalEsperado = BigDecimal.ZERO;
			totalEsperado = MoneyHelper.sumar(totalEsperado, MoneyHelper.sumar(netoSinIVA, montoIVA));
			totalEsperado = MoneyHelper.sumar(totalEsperado, MoneyHelper.sumar(noGravadoExento, percepciones));
			if (!MoneyHelper.equals(totalEsperado, total)) {
				error("El monto total no se corresponde con la suma de los demas montos");				
			} else {
				CompraAProveedorInmutable compra = new CompraAProveedorInmutable(fecha, proveedor, numero, netoSinIVA, montoIVA, noGravadoExento, percepciones, total);
				HibTransaccionConProveedorRepository repoTran = new HibTransaccionConProveedorRepository();
				repoTran.makePersistent(compra);
				setResponsePage(new CCProveedorPage(proveedor));
			}
		}
	}

}
