package org.ziggurat.fenix.web.proveedores;

import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;
import org.apache.wicket.extensions.markup.html.form.DateTextField;
import org.apache.wicket.extensions.yui.calendar.DatePicker;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.ziggurat.fenix.common.database.hibernate.HibChequesRepository;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.modelo.proveedores.PagoAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.common.modelo.transacciones.Cheque;
import org.ziggurat.fenix.common.modelo.transacciones.EstadoDeCheque;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.web.cheques.NuevoCheque;

public class NuevoPagoConCheque extends AbstractProveedoresPage {

	private Proveedor proveedor;
	private static Logger log = Logger.getLogger("fenix");
	
	public NuevoPagoConCheque(Proveedor proveedor) {
		log.info("Entrando a pagina de Nuevo pago con cheque" + proveedor.getNombre());
		this.proveedor = proveedor;
		add(new NuevoPagoForm("nuevoPagoForm"));
		add(new Label("nombreProveedor", proveedor.getNombre()));
	}

	private class NuevoPagoForm extends Form {
		
		private Date fecha = Calendar.getInstance().getTime();
		private Cheque cheque;
		private String notas;
		private String nroRecibo;
		
		public NuevoPagoForm(String id) {
			super(id);

			HibChequesRepository repoCheques = new HibChequesRepository();
			CompoundPropertyModel model = new CompoundPropertyModel(this);
			this.setModel(model);

			DateTextField dateTextField = new DateTextField("fecha", FenixFormat.DATE_FORMAT.toPattern());
			dateTextField.setRequired(true);
			dateTextField.add(new DatePicker());
			add(dateTextField);
			
			DropDownChoice cheque = new DropDownChoice("cheque", repoCheques.getChequesPorEstado(EstadoDeCheque.EN_POSESION_DE_FENIX));			
			cheque.setRequired(true);
			add(cheque);
			
			add(new Link("nuevoChequeLink") {
				@Override
				public void onClick() {
					setResponsePage(NuevoCheque.class);
				}
			});
			
			/*TextField monto = new TextField("monto") {
				@Override
				public IConverter getConverter(Class type) {
					return new FenixBigDecimalConverter();
				}
				
			};*/
			
			TextField nroRecibo = new TextField("nroRecibo");
			add(nroRecibo);
			
			TextField notas = new TextField("notas");
			add(notas);

			add(new FeedbackPanel("feedback"));
		}

		@Override
		protected void onSubmit() {
			HibTransaccionConProveedorRepository repoTran = new HibTransaccionConProveedorRepository();
			HibChequesRepository repoCheque = new HibChequesRepository();
			PagoAProveedorInmutable pago = new PagoAProveedorInmutable(proveedor, fecha, cheque, notas, nroRecibo);
			/*cheque.setEstado(EstadoDeCheque.ENTREGADO_A_PROVEEDOR);
			repoCheque.makePersistent(cheque);*/
			repoTran.makePersistentAndUpdateCheque(pago);
			setResponsePage(new CCProveedorPage(proveedor));
		}

	}

}
