package org.ziggurat.fenix.web.resumenes;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.model.Model;
import org.ziggurat.fenix.common.database.hibernate.HibCategoriasDeProveedoresRepository;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.database.hibernate.HibernateUtil;
import org.ziggurat.fenix.common.modelo.proveedores.CategoriaDeProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.LibroDeIvaCompras;

public class LibroDeIvaComprasPage extends AbstractResumenesPage {
	
	private static Logger log = Logger.getLogger("fenix");
	LibroDeIvaCompras libro;
	
	public LibroDeIvaComprasPage(final int mes, final int ano) {
		log.info("Entrando a la pagina de libro de iva de compras" + mes + " " + ano);
		setUpPage(mes, ano, null);
	}
	
	public LibroDeIvaComprasPage(int mes, int ano, CategoriaDeProveedor tipo) {		
		log.info("Entrando a la pagina de libro de iva de compras" + mes + " " + ano);
		setUpPage(mes, ano, tipo);		
	}
	
	public LibroDeIvaComprasPage(CategoriaDeProveedor categoria) {		
		this(Calendar.getInstance().get(Calendar.MONTH)+1, Calendar.getInstance().get(Calendar.YEAR), categoria);
	}
	
	public void setUpPage(final int mes, final int ano, final CategoriaDeProveedor categoria) {
		HibernateUtil.beginTransaction();		
		HibTransaccionConProveedorRepository repo = new HibTransaccionConProveedorRepository();
		
		Calendar cal  = new GregorianCalendar(ano, mes -1, 1);
		String mesString = cal.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		if (categoria == null) {
			libro = repo.getLibroDeIvaCompras(mes, ano);
			add(new Label("mes", " del mes de " + mesString + " de " + libro.getAno()));
		} else {
			libro = repo.getLibroDeIvaCompras(mes, ano, categoria);
			add(new Label("mes", "de " + categoria.getNombre() + " del mes de " + mesString + " de " + libro.getAno()));
		}
		
		add(new SelectCategoriaForm("selectCategoriaForm"));		
		add(new LibroDeIvaPanel("libroIvaPanel", libro));
		HibernateUtil.commitTransaction();
	}
	
	private class SelectCategoriaForm extends Form<CategoriaDeProveedor> {

		private CategoriaDeProveedor categoria;
		
		public SelectCategoriaForm(String id) {
			super(id);
			HibCategoriasDeProveedoresRepository repoCat = new HibCategoriasDeProveedoresRepository();
			DropDownChoice<CategoriaDeProveedor> selectCategoria = new DropDownChoice<CategoriaDeProveedor>("selectCategoria", new Model<CategoriaDeProveedor>(categoria),  repoCat.findAll()) {
				@Override
				protected void onSelectionChanged(CategoriaDeProveedor newSelection) {
					setResponsePage(new LibroDeIvaComprasPage(libro.getMes(), libro.getAno(), newSelection));
				}
				
				@Override
				protected boolean wantOnSelectionChangedNotifications() {
					return true;
				}
			};
			add(selectCategoria);
			add(new Link<LibroDeIvaComprasPage>("linkLibroDeIvaGeneral") {

				@Override
				public void onClick() {
					setResponsePage(new LibroDeIvaComprasPage(libro.getMes(), libro.getAno()));				
				}
				
			});
		}

		/**
		 * 
		 */
		private static final long serialVersionUID = -5499431491724347895L;
		
	}
	
}
