package org.ziggurat.fenix.web.resumenes;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.database.hibernate.HibernateUtil;
import org.ziggurat.fenix.common.modelo.proveedores.CategoriaDeProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.CompraAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.LibroDeIvaCompras;
import org.ziggurat.fenix.common.util.FenixFormat;

public class LibroDeIvaPanel extends Panel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6461332522872403538L;

	CategoriaDeProveedor categoria = null;
	
	public LibroDeIvaPanel(String id, final LibroDeIvaCompras libro, CategoriaDeProveedor categoria) {
		super(id);
		this.categoria = categoria;
		setUpPage(libro);
	}
	
	public LibroDeIvaPanel(String id, final LibroDeIvaCompras libro) {
		super(id);
		setUpPage(libro);
	}

	private void setUpPage(final LibroDeIvaCompras libro) {
		HibernateUtil.beginTransaction();		
		//HibTransaccionConProveedorRepository repo = new HibTransaccionConProveedorRepository();		
		final List<CompraAProveedorInmutable> facturas = libro.getFacturas();
		
		final DataView<CompraAProveedorInmutable> dataView = new DataView<CompraAProveedorInmutable>("view", new ListDataProvider<CompraAProveedorInmutable>(facturas)) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(final Item<CompraAProveedorInmutable> item) {
				final CompraAProveedorInmutable factura = item.getModelObject();
				item.add(new Label("fecha", FenixFormat.getFecha(factura.getFecha())));
				item.add(new Label("comprobante", factura.getNumero()));
				item.add(new Label("proveedor", factura.getProveedor().getNombre()));
				item.add(new Label("CUIT", factura.getProveedor().getCUIT()));
				item.add(new Label("netoSinIva", FenixFormat.getMoneda(factura.getNetoSinIVA())));
				item.add(new Label("iva", FenixFormat.getMoneda(factura.getMontoIVA())));
				item.add(new Label("noGravadoExento", FenixFormat.getMoneda(factura.getNoGravadoExento())));
				item.add(new Label("percepciones", FenixFormat.getMoneda(factura.getPercepciones())));
				item.add(new Label("total", FenixFormat.getMoneda(factura.getMonto())));
				item.add(new AttributeModifier("class", true, new AbstractReadOnlyModel<String>() {
					public String getObject() {
						return (item.getIndex() % 2 == 1) ? "even" : "odd";
					}
				}));
			}
		};
		
		add(dataView);
		add(new Label("netoSinIvaTotal", FenixFormat.getMoneda(libro.getNetoSinIVA())));
		add(new Label("ivaTotal", FenixFormat.getMoneda(libro.getMontoIVA())));
		add(new Label("noGravadoExentoTotal", FenixFormat.getMoneda(libro.getNoGravadoExento())));
		add(new Label("percepcionesTotal", FenixFormat.getMoneda(libro.getPercepciones())));
		add(new Label("totalTotal", FenixFormat.getMoneda(libro.getTotal())));
		
		//Calendar cal  = new GregorianCalendar(ano, mes -1, 1);
		Calendar cal  = new GregorianCalendar(libro.getAno(), libro.getMes()-1, 1);
		String mesString = cal.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());		
		//add(new Label("mes", mesString + " " + libro.getAno()));
		
		
		//final Calendar calAnterior  = new GregorianCalendar(ano, mes -1, 1);
		final Calendar calAnterior  = new GregorianCalendar(libro.getAno(), libro.getMes() -1, 1);
		calAnterior.add(Calendar.MONTH, -1);
		mesString = calAnterior.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Link<LibroDeIvaCompras>("mesAnteriorLink") {
			/**
			 * 
			 */
			private static final long serialVersionUID = -5625485630477637507L;

			@Override
			public void onClick() {
				if (categoria == null) {
					setResponsePage(new LibroDeIvaComprasPage(calAnterior.get(Calendar.MONTH) +1, calAnterior.get(Calendar.YEAR)));
				} else {
					setResponsePage(new LibroDeIvaComprasPage(calAnterior.get(Calendar.MONTH) +1, calAnterior.get(Calendar.YEAR), categoria));
				}
				
			}
		}.add(new Label("mesAnterior", mesString)));
		
		//final Calendar calSiguiente  = new GregorianCalendar(ano, mes -1, 1);
		final Calendar calSiguiente  = new GregorianCalendar(libro.getAno(), libro.getMes() -1, 1);
		calSiguiente.add(Calendar.MONTH, 1);
		mesString = calSiguiente.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Link<LibroDeIvaCompras>("mesSiguienteLink") {
			/**
			 * 
			 */
			private static final long serialVersionUID = -8345434908924190635L;

			@Override
			public void onClick() {
				if (categoria == null) {
					setResponsePage(new LibroDeIvaComprasPage(calSiguiente.get(Calendar.MONTH) +1, calSiguiente.get(Calendar.YEAR)));
				} else {
					setResponsePage(new LibroDeIvaComprasPage(calSiguiente.get(Calendar.MONTH) +1, calSiguiente.get(Calendar.YEAR), categoria));
				}
				
			}
		}.add(new Label("mesSiguiente", mesString)));
		HibernateUtil.commitTransaction();
	}
	
}
