package org.ziggurat.fenix.web.resumenes;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.modelo.proveedores.PagoAProveedorInmutable;
import org.ziggurat.fenix.common.util.FenixFormat;
import org.ziggurat.fenix.web.proveedores.CCProveedorPage;
import org.ziggurat.fenix.web.proveedores.LineaDeCCPagoChequePanel;
import org.ziggurat.fenix.web.proveedores.LineaDeCCPagoEfectivoPanel;

public class ListadoDePagosPage extends AbstractResumenesPage {

	private static Logger log = Logger.getLogger("fenix");
	
	public ListadoDePagosPage(int mes, int ano) {
		log.info("Entrando a pagina de Listado de pagos " + mes + " " + ano);
		HibTransaccionConProveedorRepository repo = new HibTransaccionConProveedorRepository();
		List<PagoAProveedorInmutable> pagos = repo.getPagosDelMes(mes, ano);

		final DataView<PagoAProveedorInmutable> dataView = new DataView<PagoAProveedorInmutable>("view", new ListDataProvider(pagos)) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(final Item<PagoAProveedorInmutable> item) {
				final PagoAProveedorInmutable pago = item.getModelObject();
				item.add(new Label("fecha", FenixFormat.getFecha(pago.getFecha())));
				item.add(new Link("proveedorLink") {
					@Override
					public void onClick() {
						setResponsePage(new CCProveedorPage(pago.getProveedor()));
					}
				}.add(new Label("proveedor", pago.getProveedor().getNombre())));				
				if (pago.getCheque() != null) {
					item.add(new LineaDeCCPagoChequePanel("linkDetalles", pago));					
				} else {
					item.add(new LineaDeCCPagoEfectivoPanel("linkDetalles"));
				}
				item.add(new Label("monto", FenixFormat.getMoneda(pago.getMonto())));
				item.add(new Label("notas", StringUtils.abbreviate(pago.getNotas(), 10)));
				item.add(new AttributeModifier("class", true, new AbstractReadOnlyModel() {
					public Object getObject() {
						return (item.getIndex() % 2 == 1) ? "even" : "odd";
					}
				}));
			}
		};
		
		add(dataView);

		Calendar cal = new GregorianCalendar(ano, mes - 1, 1);
		String mesString = cal.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Label("mes", mesString + " " + ano));

		final Calendar calAnterior = new GregorianCalendar(ano, mes - 1, 1);
		calAnterior.add(Calendar.MONTH, -1);
		mesString = calAnterior.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Link("mesAnteriorLink") {
			@Override
			public void onClick() {
				setResponsePage(new ListadoDePagosPage(calAnterior.get(Calendar.MONTH) + 1, calAnterior.get(Calendar.YEAR)));
			}
		}.add(new Label("mesAnterior", mesString)));

		final Calendar calSiguiente = new GregorianCalendar(ano, mes - 1, 1);
		calSiguiente.add(Calendar.MONTH, 1);
		mesString = calSiguiente.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Link("mesSiguienteLink") {
			@Override
			public void onClick() {
				setResponsePage(new ListadoDePagosPage(calSiguiente.get(Calendar.MONTH) + 1, calSiguiente.get(Calendar.YEAR)));
			}
		}.add(new Label("mesSiguiente", mesString)));
	}

}
