package org.ziggurat.fenix.web.resumenes;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.markup.repeater.data.ListDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.ziggurat.fenix.common.database.hibernate.HibTransaccionConProveedorRepository;
import org.ziggurat.fenix.common.modelo.proveedores.ResumenMensual;
import org.ziggurat.fenix.common.modelo.proveedores.ResumenPorTipo;
import org.ziggurat.fenix.common.util.FenixFormat;

public class ResumenMensualPage extends AbstractResumenesPage {

	private static Logger log = Logger.getLogger("fenix");

	public ResumenMensualPage(int mes, int ano) {
		log.info("Entrando a pagina de resumen mensual " + mes + " " + ano);
		HibTransaccionConProveedorRepository repo =  new HibTransaccionConProveedorRepository();
		final ResumenMensual resumen = repo.getResumenProveedores(mes, ano);
		
		final DataView dataView = new DataView("view", new ListDataProvider(resumen.getResumenPorTipo())) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(final Item item) {
				final ResumenPorTipo resumen = (ResumenPorTipo) item.getModelObject();
				item.add(new Link("tipoLink") {
					@Override
					public void onClick() {
						setResponsePage(new LibroDeIvaComprasPage(resumen.getTipo()));
					}
				}.add(new Label("tipo", resumen.getTipo().toString())));

				item.add(new Label("netoSinIva", FenixFormat.getMoneda(resumen.getNetoSinIVA())));
				item.add(new Label("iva", FenixFormat.getMoneda(resumen.getMontoIVA())));
				item.add(new Label("noGravadoExento", FenixFormat.getMoneda(resumen.getNoGravadoExento())));
				item.add(new Label("percepciones", FenixFormat.getMoneda(resumen.getPercepciones())));
				item.add(new Label("total", FenixFormat.getMoneda(resumen.getTotalFacturas())));
				item.add(new Label("pagos", FenixFormat.getMoneda(resumen.getTotalPagos())));
				item.add(new Label("saldo", FenixFormat.getMoneda(resumen.getSaldo())));
				item.add(new AttributeModifier("class", true, new AbstractReadOnlyModel() {
					public Object getObject() {
						return (item.getIndex() % 2 == 1) ? "even" : "odd";
					}
				}));
			}
		};
		add(new Label("netoSinIvaTotal", FenixFormat.getMoneda(resumen.getNetoSinIVA())));
		add(new Label("ivaTotal", FenixFormat.getMoneda(resumen.getMontoIVA())));
		add(new Label("noGravadoExentoTotal", FenixFormat.getMoneda(resumen.getNoGravadoExento())));
		add(new Label("percepcionesTotal", FenixFormat.getMoneda(resumen.getPercepciones())));
		add(new Label("totalTotal", FenixFormat.getMoneda(resumen.getTotalCompras())));
		add(new Label("pagosTotal", FenixFormat.getMoneda(resumen.getTotalPagos())));
		add(new Label("saldoTotal", FenixFormat.getMoneda(resumen.getSaldo())));		
		add(dataView);
		
		Calendar cal  = new GregorianCalendar(ano, mes -1, 1);
		String mesString = cal.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());		
		add(new Label("mes", mesString + " " + ano));
		
		
		final Calendar calAnterior  = new GregorianCalendar(ano, mes -1, 1);
		calAnterior.add(Calendar.MONTH, -1);
		mesString = calAnterior.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Link("mesAnteriorLink") {
			@Override
			public void onClick() {
				setResponsePage(new ResumenMensualPage(calAnterior.get(Calendar.MONTH) +1 , calAnterior.get(Calendar.YEAR)));
			}
		}.add(new Label("mesAnterior", mesString)));
		
		final Calendar calSiguiente  = new GregorianCalendar(ano, mes -1, 1);
		calSiguiente.add(Calendar.MONTH, 1);
		mesString = calSiguiente.getDisplayName(Calendar.MONTH, Calendar.LONG, Locale.getDefault());
		add(new Link("mesSiguienteLink") {
			@Override
			public void onClick() {
				setResponsePage(new ResumenMensualPage(calSiguiente.get(Calendar.MONTH) +1, calSiguiente.get(Calendar.YEAR)));
			}
		}.add(new Label("mesSiguiente", mesString)));
		
		/*add(new Link("mostrarGrafico") {
			@Override
			public void onClick() {
				setResponsePage(new GraficoResumenMensual(resumen));
			}
		});*/
	}
}
